#include "parameters.h"


using namespace std;

namespace cvflannIndex {

string usage_string = "Clustering Indexing Application";
po::variables_map params;
po::options_description help("Help");
po::options_description args("General Arguments");
po::options_description formats("File Formats");
po::options_description processing("Processing");
po::options_description clustering("Clustering");
po::options_description cmdline_options;

void print_usage() {
    cout << usage_string << endl << cmdline_options;
}

void getParameters (int ac, char* av[]) {
    help.add_options()
            ("help,h", "Show this message")
    ;
    args.add_options()
            // -- Parameters
            ("feature,f",
                    po::value<string>(), //->default_value("esp.feature.hdf5"),
                    "Feature file (Text, HDF5)")
            ("index,i",
                    po::value<string>(), //->default_value("esp.feature.index.hdf5"),
                    "Index file (HDF5)")
            ("cluster,c",
                    po::value<string>(), //->default_value("esp.feature.clusters.hdf5"),
                    "Cluster file (HDF5)")
            ("directory,d",
                    po::value<string>(),
                    "Target directory (PGM images, feature files etc)")
            ("image,m",
                    po::value<string>(),
                    "Image file (PGM image)")
            ("filein1,1",
                    po::value<string>(),
                    "File (Text)")
            ("filein2,2",
                    po::value<string>(),
                    "File (Text)")
            ("fileout,o",
                    po::value<string>(),
                    "File (Text)")
    ;
    formats.add_options()
            // -- Actions
            ("convert-to-hdf5",
                    "Convert <feature> file to HDF5 format, store as <feature>.hdf5")
    ;
    processing.add_options()
            // -- Actions
            ("extract-features",
                    "Extract features from images in <directory> folder, and store them in separate files")
            ("merge-features",
                    "Merge feature files from <directory> folder, and store them in <feature> file")
            ("query-image",
                    "Query image located at <image> file")
            ("concat-files",
                    "Concatanate files <file1> and <file2> into new file <fileout>")
            ("count-lines",
                    "Count the number of lines in <file1>")
    ;
    clustering.add_options()
            ("build-clusters",
                    "Build K-Means clusters from <feature>, save to <clusters>")
            ("build-cluster-index",
                    "Load clusters from <cluster> or build from clusters extracted from <feature>, then"
                    "index the clusters, save the index to <index>")
            ("load-cluster-index",
                    "Load clusters from <clusters> or build from <feature>, then"
                    "load the pre-indexed clusters from <index> and <cluster>")
            ("num-clusters,n",
                    po::value<int>()->default_value(1000),
                    "Number of clusters")
            ("num-iters,t",
                    po::value<int>()->default_value(15),
                    "K-Means iterations")
    ;

    cmdline_options.add(help).add(args).add(formats).add(processing).add(clustering);
    po::store(po::parse_command_line(ac, av, cmdline_options),  params);
    po::notify(params);
}

}
