#ifndef CAMERA_H
#define CAMERA_H

//#include "pwc-ioctl.h"
#include <fcntl.h>
#include <stdio.h>
#include <string>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <unistd.h>
#include <linux/videodev.h>
#include <iostream>
#include <fstream>
#include "cameraConf.h"

using namespace std;

//! Camera class
/*!
        Camera class handles the interactions with the camera
        and enables the user to set camera parameters
*/
class Camera{
private:
    CameraConf conf;

    struct video_mbuf* vbuf;
    struct video_mmap* vmmap;

    int dev;
    string deviceName;
    unsigned char* memoryMap;
    unsigned char* v4l_convert_buf;
    int bufferIndex;

    //! Convert the image in the buffer from YUV format to RGB format
    unsigned char* convert_yuv2rgb();

    //! Fill out the fields and request capture to each buffer except the last one
    void updateVideommap();

    void updatePictureSettings();
    void updateWindowSettings();
    void updateChannelSettings();

public:
    //! Constructor
    /*! Requires the device name as the parameter */
    // Defined as explicit because otherwise the class will be implicity converted to the parameter (in this case string) type
    explicit Camera(string devName);

    //! Desctructor
    ~Camera();

    //! Grab a frame and store it in the buffer
    unsigned char* getFrame();

    //! Save an image in *ppm format
    int saveImagePPM(unsigned char* img, string filename);

    CameraConf Configuration() { return conf; }

    int Width();
    int Height();
    int Brightness();
    int Hue();
    int Colour();
    int Contrast();
    int Whiteness();
    int Depth();
    int Palette();

    void SetFrameSize(int w, int h);
    void Brightness(int);
    void Hue(int);
    void Colour(int);
    void Contrast(int);
    void Whiteness(int);
    void Depth(int);
    void Palette(int);


    //! Returns current camera parameters
    CameraConf getCameraConf();

    //! Sets camera parameters
    void setCameraConf(const CameraConf&);

    //! Load the config file from file
    int loadConf(string filename);

    //! Save the current configuration to the file
    int saveConf(string filename);

};

#endif
