#include "dictionary.h"
#include <sstream>
#include <opencv2/core/core.hpp>

using namespace std;
using namespace converter;
using namespace boost;


Dictionary::Dictionary(string clusterFile, string indexFile)
{

    index = new cvflannIndex::Index<FT,CT>();
    index->loadClusters(clusterFile);
    index->loadClusterIndex(indexFile);


    /*
    clusters.data = NULL;
    clusterIndex = NULL;

    cout << "Loading clusters from " << clusterfile << endl;
    flamm::load_from_file(clusters, clusterfile,"dataset");

    cout << "Loading cluster index from " << indexfile << endl;
    clusterIndex = flamm_load_index((char*) indexfile.c_str(), clusters.data, clusters.rows, clusters.cols);
    if (!clusterIndex) {
        cout << "ERROR: Could not load index file" << endl;
       exit(0);
    }

    indexParams = flamm::DEFAULT_FLAMM_PARAMETERS;
    indexParams.algorithm = flamm::KDTREE;
    indexParams.trees = 8;
*/
}

Dictionary::~Dictionary()
{
    cout<<"dictionary destructor"<<endl;

    clear();
    delete index;
    index = NULL;
//    clusters.free();
//    delete clusterIndex;
}

void Dictionary::clear() {
    BOOST_FOREACH(InvIndex::value_type &w, invIndex) {
        BOOST_FOREACH(OccuranceList::value_type &o, w.second) {
            o.second.clear();
        }
        w.second.clear();
    }
    invIndex.clear();
    //dict.clear();
}


bool Dictionary::isEmpty() {
    //return dict.empty();
    return invIndex.empty();
}

int Dictionary::size() {
    //return dict.size();
    return invIndex.size();
}

int Dictionary::numWords() {
    return index->getClusters()->rows;
}


void Dictionary::queryDictionary(Image *img, Mat &indices, Mat &distances) {

    int N = (int)img->features.size();

    Mat query(N,appConf.FEATUREDIM,cv::flann::CvType<CT>().type());
    for(int i=0; i<N; i++) {
        for(int j=0; j<appConf.FEATUREDIM; j++) {
            query.at<CT>(i,j) = (CT)img->features[i]->Descr().Data()[j];
        }
    }

    int nn = 1;
    indices.create(query.rows,nn,CV_32S);
    distances.create(query.rows,nn,CV_32F);

    index->getIndex()->knnSearch(query, indices, distances, nn, cv::flann::SearchParams(128));

/*
    //convert myfeature to flann::Matrix<feat>
    flamm::Matrix<float> queryfeatures;
    queryfeatures.rows = N;
    queryfeatures.cols = FEATUREDIM;
    queryfeatures.data = new float[N*FEATUREDIM];

    for(int i=0; i<N; i++) {
        Feat *f = img->features[i];
        for(int j=0; j<FEATUREDIM; j++) {
            queryfeatures.data[j+i*FEATUREDIM] = f->Descr().Data()[j];
        }
    }

    // -- Compute nearest neighbors
    //cout << "Computing nearest neighbors" << endl;
    flamm_find_nearest_neighbors_index(clusterIndex, queryfeatures.data,
            queryfeatures.rows, indices, distances, 1, &indexParams);

    for(int i=0; i<N; i++) {
        //cout<<"word inserted to the list in the image: "<<indices[i]<<endl;
        img->words.push_back(indices[i]);
    }
    delete[] queryfeatures.data;
    */
}


void Dictionary::addImage(Image *img) {
    int imgIndex = img->getIndex();

    bool inserted = false;
    if(img->words.size()>0)
        inserted = true;

    // This if(inserted) check is added for performance purposes...
    // It comes into play when an image inserted into some other map is being inserted into this one
    // (word list is filled with the word indices pointing to words in its own dictionary)
    // ASSUMING WE KEEP DICTIONARIES FIXED there is no need to query the dictionary again
    // However, this check can be removed to generalize the process
    if(!inserted) {

        if(img->features.size()==0)
            img->extractFeatures();

        int N = (int)img->features.size();


        // -- Compute nearest neighbors
        Mat indices;
        Mat distances;
        queryDictionary(img, indices, distances);

        // Add to the matched word's occurances to inverted index
        for(int i=0; i<N; i++) {

            double x = ((feat::FeatProp *)(img->features[i]->prop))->Location().X();
            double y = ((feat::FeatProp *)(img->features[i]->prop))->Location().Y();
            cv::Point2d pnew(x,y);

            int wordIndex = indices.at<int>(i,0);
            invIndex[wordIndex][imgIndex].push_back(pnew);
            img->words.push_back(make_pair(wordIndex,pnew));
        }
    }
    else {      
        // Add to the matched word's occurances to inverted index
        for(int i=0; i<img->words.size(); i++) {
            invIndex[img->words[i].first][imgIndex].push_back(img->words[i].second);
        }
    }
}


// Assuming matches is empty..
// Property pointers in matches point to both Image->feature[i].prop and property in dictionary
void Dictionary::findMatches(Image *img, ZointMatchMap &matches)
{
    int N = (int)img->features.size();
    int M = (int)img->words.size();

    if(M>0) {
        // directly use the word indices to find the matches

        for(int i=0; i<M; i++) {
            BOOST_FOREACH(OccuranceList::value_type &o, invIndex[img->words[i].first]) {
                BOOST_FOREACH(cv::Point2d &point, o.second) {
                    matches[o.first].first.push_back(img->words[i].second);
                    matches[o.first].second.push_back(point);
                }
            }
        }
    }
    else {
        // -- Compute nearest neighbors
        Mat indices;
        Mat distances;
        queryDictionary(img, indices, distances);

        // for each feature
        for(int i=0; i<N; i++) {

            double x1 = ((FeatProp *)(img->features[i]->prop))->Location().X();
            double y1 = ((FeatProp *)(img->features[i]->prop))->Location().Y();
            cv::Point2d p1(x1,y1);

            if(invIndex.find(indices.at<int>(i,0)) != invIndex.end()) {
                BOOST_FOREACH(OccuranceList::value_type &o, invIndex[indices.at<int>(i,0)]) {
                    BOOST_FOREACH(cv::Point2d &point, o.second) {
                        cv::Point2d p2(point.x,point.y);
                        matches[o.first].first.push_back(p1);
                        matches[o.first].second.push_back(p2);
                    }
                }
            }
        }
    }
}





void Dictionary::print() {
    cout << "Printing the occurances in the dictionary of size: "<<invIndex.size() << endl;
    BOOST_FOREACH(InvIndex::value_type &ol, invIndex) {
        cout<<"Word: "<<ol.first<<endl;
        BOOST_FOREACH(OccuranceList::value_type &o, ol.second) {
            BOOST_FOREACH(cv::Point2d &point, o.second) {
                cout<<"Appeared in: "<<o.first<<" at "<<point.x<<" "<<point.y<<endl;
            }
        }
    }
}


// This function actually writes inverted index file
void Dictionary::writeDictionary(string filename) {
    writeInvIndex(filename);
}

void Dictionary::writeInvIndex(string filename) {

    Debug(1)<<"Writing database to file: "<<filename<<endl;

    ofstream myfile;
    myfile.open(filename.c_str(),ios::out);
    if(myfile.is_open()) {
        myfile << "Occupied dictionary size: " << invIndex.size() << endl;

        BOOST_FOREACH(InvIndex::value_type &ol, invIndex) {
            myfile << "Word: " << ol.first << endl;
            BOOST_FOREACH(OccuranceList::value_type &o, ol.second) {
                BOOST_FOREACH(cv::Point2d &point, o.second) {
                    myfile << "Appeared in image: " << o.first<<" at "<<point.x<<" "<<point.y<<endl;
                }
            }
        }
        myfile.close();
    }
}



void Dictionary::readDictionary(string filename, ImageTable &imageLookup) {
    readInvIndex(filename, imageLookup);
}

void Dictionary::readInvIndex(string filename, ImageTable &imageLookup) {

    Debug(1)<<"Reading database from file: "<<filename<<endl;

    ifstream myfile(filename.c_str());
    if(myfile.is_open()) {
        clear();

        string line,temp;
        istringstream parse;
        getline(myfile,line); //  number of words in the dict file

        Debug(0) << "\tStarting reading " << line << " words" << endl;


        int wordIndex = -1;
        int imageIndex = -1;

        while(getline(myfile,line)) {
            if(!converter::isEmpty(line)) {

                parse.clear();
                parse.str(line);
                parse >> temp;

                if(temp=="Word:") {
                    parse >> wordIndex;
                    //cout << wordIndex << endl;
                }
                else if(temp=="Appeared") {
                    if(wordIndex==-1) {
                        cout<<"ERROR: Occurance with no index detected" <<endl;
                        exit(0);
                    }
                    parse >> temp >> temp >> imageIndex;
                    parse >> temp;
                    double x,y;
                    parse >> x >> y;
                    cv::Point2d p(x,y);
                    invIndex[wordIndex][imageIndex].push_back(p);
                    imageLookup[imageIndex]->words.push_back(make_pair(wordIndex,p));
                }
            } //endif
        } //endwhile

        //cout<<"db done"<<endl;
        myfile.close();
    }
}


