#include "imagecomparisonwidget.h"
#include <QtGui>

#include <iostream>
#include <map>
#include <set>
using namespace std;


ImageComparisonWidget::ImageComparisonWidget(Image *img1,Image *img2, int orientation, Edge* e, SimMap *m, QWidget *parent)
    : QWidget(parent)
{
    image1 = new Image(img1);
    image2 = new Image(img2);
    edge = e;
    mymap = m;

    confwin = NULL;
    subwin = NULL;
    confsubwin = NULL;

    numMatches = 0;

    if(orientation == 0)
        isVertical = false;
    else
        isVertical = true;

    pix1 = new QPixmap;
    pix2 = new QPixmap;
    pix1_org = new QPixmap;
    pix2_org = new QPixmap;

    image1Label = new QLabel(this);
    image2Label = new QLabel(this);
    //image1Label->setSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored);
    //image2Label->setSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored);
    image1Label->setScaledContents(true);
    image2Label->setScaledContents(true);

    statusBar = new QStatusBar;
    statusBar->setSizeGripEnabled(false);

    QBoxLayout *box;
    if(isVertical)
        box = new QVBoxLayout;
    else
        box = new QHBoxLayout;
    box->addWidget(image1Label);
    box->addWidget(image2Label);
    box->setSpacing(0);
    box->setContentsMargins(0,0,0,0);

    QVBoxLayout *vbox = new QVBoxLayout;
    vbox->addLayout(box);
    vbox->addWidget(statusBar);
    vbox->setSpacing(0);
    vbox->setContentsMargins(0,0,0,0);
    setLayout(vbox);
    layout()->setSizeConstraint(QLayout::SetFixedSize);


    infoAct = new QAction(tr("&Info"), this);
    infoAct->setShortcut(tr("Ctrl+I"));
    infoAct->setEnabled(true);
    connect(infoAct, SIGNAL(triggered()), this, SLOT(showInfo()));

    confAct = new QAction(tr("&Configuration window"), this);
    confAct->setShortcut(tr("Ctrl+C"));
    confAct->setEnabled(true);
    connect(confAct, SIGNAL(triggered()), this, SLOT(showConfWindow()));

    openImages();

    if(edge==NULL && mymap==NULL)
        drawMatches(1);
    else
        drawMatches();
}

void ImageComparisonWidget::closeEvent(QCloseEvent *)
{
    //cout<<"close event for imagecomparisonwidget"<<endl;
    emit closeSignal();
    if(image1!=NULL) {
        delete image1;
        image1=NULL;
    }
    if(image2!=NULL) {
        delete image2;
        image2=NULL;
    }
}

void ImageComparisonWidget::close()
{
    //cout<<"image WIDGET close"<<endl;
    subwin->mdiArea()->removeSubWindow(subwin);
    subwin->close();
    //cout<<"image WIDGET close DONE"<<endl;
}


void ImageComparisonWidget::killConfWindow()
{
    confwin = NULL;
    //subwin->mdiArea()->setActiveSubWindow(subwin);
    confAct->setEnabled(true);
}


void ImageComparisonWidget::setSubWindowPtr(QMdiSubWindow *s)
{
    subwin = s;    
    subwin->layout()->setSizeConstraint(QLayout::SetFixedSize);
}


void ImageComparisonWidget::contextMenuEvent(QContextMenuEvent *event) {
    QMenu contextMenu(this);
    contextMenu.addAction(confAct);
    contextMenu.addAction(infoAct);
    contextMenu.exec(event->globalPos()+QPoint(3,3));
}

void ImageComparisonWidget::showInfo() {
    ostringstream s;
    s << "Image (";
    if(image1->getIndex()<0)    s << image1->getTitle();
    else                        s << image1->getIndex();
    s << "): " << image1->features.size() << " features" <<endl;

    s << "Image (";
    if(image2->getIndex()<0)    s << image2->getTitle();
    else                        s << image2->getIndex();
    s << "): " << image2->features.size() << " features" <<endl;

    s << "Total matches: " << numMatches << endl;
    QMessageBox::about(this, tr("Info"),QString(s.str().c_str()));
}


void ImageComparisonWidget::showConfWindow() {
    confwin = new ImageComparisonConfig();

    if(edge==NULL && mymap==NULL)
        confwin->switchToPairOnlyMode();

    connect(confwin,SIGNAL(configChanged()),this,SLOT(updateDisplay()));
    connect(this,SIGNAL(closeSignal()),confwin,SLOT(close()));
    connect(confwin,SIGNAL(closeSignal()),this,SLOT(killConfWindow()));

    if(subwin!=NULL) {
        confsubwin = subwin->mdiArea()->addSubWindow(confwin);
        confwin->setSubWindowPtr(confsubwin);

        QPoint pos = subwin->pos();
        pos.setX(pos.x() + subwin->size().width());
        confsubwin->move(pos);
    }

    confwin->show();
    confAct->setEnabled(false);
}

bool ImageComparisonWidget::openImages() {

    if(image1==NULL || image2==NULL) {
        cout<<"ERROR: NULL image is loaded\n"<<endl;
        return false;
    }

    if(image1->isEmpty()) {
        if(image1->loadImageFromFile()!=0) {
            cout<<"ERROR: Empty image, loading failed"<<endl;
            return false;
        }
    }
    if(image2->isEmpty()) {
        if(image2->loadImageFromFile()!=0) {
            cout<<"ERROR: Empty image, loading failed"<<endl;
            return false;
        }
    }

    if(openImage(image1,*pix1_org) && openImage(image2,*pix2_org))
        return true;
    return false;
}


bool ImageComparisonWidget::openImage(Image *img, QPixmap &pix) {
    if(img->getImageFormat()==Image::RGB) {
        QImage image((uchar *) img->getImageData(), img->getWidth(), img->getHeight(), QImage::Format_RGB888);
        pix.convertFromImage(image);
        return true;
    }
    else if(img->getImageFormat()==Image::GRAY) {
        QImage image((uchar *) img->getImageData(), img->getWidth(), img->getHeight(), QImage::Format_Indexed8);
        QVector<QRgb> colorTable;
        for(int i=0;i<256;i++)
            colorTable.push_back(QColor(i,i,i).rgb());

        image.setColorTable(colorTable);
        pix.convertFromImage(image);
        return true;
    }
    return false;
}



// method: 0 (edge), 1(pairwise), 2(custom)
// filter: 0b00X: F matrix support
//         0b0X0: outlier detection
//         0bX00: spatial consistency
void ImageComparisonWidget::drawMatches(int method, int filter)
{
    if(image1==NULL || image2==NULL) {
        cout<<"Cannot draw matches: Images are empty."<<endl;
    }
    else {

        if(image1->features.size()==0)
            image1->extractFeatures();
        if(image2->features.size()==0)
            image2->extractFeatures();

        *pix1 = *pix1_org;
        *pix2 = *pix2_org;

        QPainter painter1(pix1);
        QPainter painter2(pix2);

        QPen Red((QColor::QColor(255,0,0)),1);
        QPen Green((QColor::QColor(0,255,0)),1);
        QPen Yellow((QColor::QColor(255,255,0)),1);
        QPen Blue((QColor::QColor(0,0,255)),1);

        int crossSize = 4;
        painter1.setPen(Yellow);
        for(unsigned int i=0; i < image1->features.size(); i++) {
            double x = ((SiftProp*)(image1->features[i]->prop))->Location().X();
            double y = ((SiftProp*)(image1->features[i]->prop))->Location().Y();
            painter1.drawLine(x-crossSize/2,y-crossSize/2,x+crossSize/2,y+crossSize/2);
            painter1.drawLine(x-crossSize/2,y+crossSize/2,x+crossSize/2,y-crossSize/2);
        }
        painter2.setPen(Yellow);
        for(unsigned int i=0; i < image2->features.size(); i++) {
            double x = ((SiftProp*)(image2->features[i]->prop))->Location().X();
            double y = ((SiftProp*)(image2->features[i]->prop))->Location().Y();
            painter2.drawLine(x-crossSize/2,y-crossSize/2,x+crossSize/2,y+crossSize/2);
            painter2.drawLine(x-crossSize/2,y+crossSize/2,x+crossSize/2,y-crossSize/2);
        }

        ZointMatchList *pml=NULL;

        if(method==0) {
            if (edge == NULL) {
                cout<<"Cannot draw matches: edge is empty"<<endl;
            }
            else {
                pml = edge->matches;
            }
        }
        else if(method==1) { // pairwise
            pml = new ZointMatchList;
            image1->matchImage(image2,*pml);
        }
        else { // method=2 (custom)
            // need initial matches first
            if(mymap!=NULL) {
                ZointMatchMap matches;
                mymap->getDictionary()->findMatches(image1, matches);

                CvMat* F = cvCreateMat(3,3,CV_64FC1);
                cv::Mat t(F);
                pml = new ZointMatchList;
                for(unsigned int j=0; j<matches[image2->getIndex()].first.size(); j++) {
                    pml->first.push_back(matches[image2->getIndex()].first[j]);
                    pml->second.push_back(matches[image2->getIndex()].second[j]);
                }

                iu::enforceFundMatConstraint(t, *pml, filter,image2->getHeight());
            }
        }

        if(pml!=NULL) {
            painter1.setPen(Blue);
            painter2.setPen(Blue);

            ZointList *pl1 = &(pml->first);
            ZointList *pl2 = &(pml->second);
            numMatches = pl1->size();

            int uniques = iu::getUniqueFeats(*pml);

            cout<<"Drawing edges: "<<numMatches<<"  uniques: "<<uniques<<endl;
            for(unsigned int i=0; i<pl1->size(); i++) {
                double x1 = (*pl1)[i].x;
                double y1 = (*pl1)[i].y;
                double x2 = (*pl2)[i].x;
                double y2 = (*pl2)[i].y;

                //cout<<x1<<" "<<y1<<" vs "<<x2<<" "<<y2<<endl;

                if(isVertical) {
                    painter1.drawLine(x1,y1,x2,y2+image1->getHeight());
                    painter2.drawLine(x1,y1-image1->getHeight(),x2,y2);
                }
                else {
                    painter1.drawLine(x1,y1,x2+image1->getWidth(),y2);
                    painter2.drawLine(x1-image1->getWidth(),y1,x2,y2);
                }
            }
        }

        if(pml!=NULL && method!=0)
            delete pml;

        image1Label->setPixmap(*pix1);
        image2Label->setPixmap(*pix2);
        image1Label->show();
        image2Label->show();
        updateStatusBar();
    }    
}

void ImageComparisonWidget::updateDisplay()
{
    int method = confwin->getMethod();
    int filter = confwin->getFilter();
    drawMatches(method,filter);
}

void ImageComparisonWidget::updateStatusBar()
{
    /*
    QString s1,s2;
    if(image1->getIndex()<0)    s1 = QString(image1->getTitle().c_str());
    else                        s1 = QString(image1->getIndex());
    if(image2->getIndex()<0)    s2 = QString(image2->getTitle().c_str());
    else                        s2 = QString(image2->getIndex());
    */
    QString message = QString("%1 matches. Feats: %2 vs %3")
            .arg(numMatches)
            .arg(image1->features.size())
            .arg(image2->features.size());


    statusBar->showMessage(message);
}

