#ifndef IMAGEUTILS_H
#define IMAGEUTILS_H

#include "defs.h"
#include <limits>
#include <vector>
#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/calib3d/calib3d.hpp>

using namespace std;
using namespace cv;

namespace iu {

    int enforceFundMatConstraint(Mat &F, ZointMatchList &pointList, int filterlevel=0b111, int img2Height=0, IplImage *img1=NULL, IplImage *img2=NULL);

    Mat calcFundamentalMat(ZointMatchList *pointList, int method=0);
    Mat calcFundamentalMat_Ransac(ZointMatchList *pointList, int numiter, int method=0, int numsamples=8, int mininliers=10);

    double calcFundMatError(Mat &F, ZointMatchList &pointList, int errMethod=0, double errThreshold=-1);
    double calcFundMatError_Resudial(Mat &F, ZointMatchList &pointList, double errThreshhold=1.5);
    double calcFundMatError_Sampsons(Mat &F, ZointMatchList &pointList, double errThreshhold=0.1);

    void removeDuplicates(ZointMatchList &pointList);
    void removeOutliers(ZointMatchList &pointList);
    void applySpatialConsistency(ZointMatchList &pointList, int imgHeight, int nn=5);

    int getUniqueFeats(ZointMatchList &pointList);

    Mat normalizeLine(Mat &L);
    void forceRank2(Mat &F);

    void visualizeMatches(IplImage *img1, IplImage *img2, ZointMatchList &pointList, string title);

    bool isOutlier(double x, double mean, double stddev, int numPoints);
}

#endif // IMAGEUTILS_H
