#include "global.h"
#include "mainwindow.h"
#include "imagewindow.h"
#include "mapwidget.h"
#include "mapmergedialog.h"
#include "imagecomparisondialog.h"
#include "imagecomparisonwidget.h"
#include "addimagesdialog.h"
#include "vertexinput.h"
#include "edgeinput.h"
#include "edgeinputtext.h"
#include <iostream>
#include <QtGui>
#include <QFileDialog>
#include <QFileInfo>
#include <QtGui>
#include <boost/thread/thread.hpp>


using namespace std;

conf::VisAppConf appConf;

MainWindow::MainWindow(QMainWindow* parent)
    : QMainWindow(parent)
{
    mdiArea  = new QMdiArea;
    setCentralWidget(mdiArea);
    connect(mdiArea,SIGNAL(subWindowActivated(QMdiSubWindow*)),this,SLOT(updateActions()));

    createActions();
    createMenus();
    createToolBars();
    createStatusBar();

    showMaximized();

    //workingDirectory = ;

    // readSettings();
    // ask the user to load the last workspace?
    //connect(visapp, SIGNAL(modified()), this, SLOT(visappModified()));

    QString conffilepath("sys/myConfiguration.conf");
    QFile conffile(conffilepath);
    if(conffile.exists()) {
        appConf.loadConf(conffilepath.toStdString());
    }

    QColor::setAllowX11ColorNames(true);
}


void MainWindow::closeEvent(QCloseEvent *event)
{
    cout<<"close event for MainWindow"<<endl;

    //    mdiArea->closeAllSubWindows();
    while(mdiArea->subWindowList().size()>0)
    {
        QMdiSubWindow *sub = mdiArea->subWindowList()[0];
        mdiArea->removeSubWindow(sub);
        sub->close();
    }

    if (mdiArea->currentSubWindow()) {
        event->ignore();
    } else {
        //writeSettings();
        event->accept();
    }
}



void MainWindow::createActions()
{
    exitAction = new QAction(tr("E&xit"),this);
    exitAction->setStatusTip(tr("Exit the application"));
    exitAction->setShortcut(tr("CTRL+Q"));
    exitAction->setIcon(QIcon(ICON_EXIT));
    connect(exitAction,SIGNAL(triggered()),this,SLOT(close()));


    // MAP TOOLBAR
    openMapAction = new QAction(tr("&Open Map"),this);
    openMapAction->setStatusTip(tr("Open Map"));
    openMapAction->setShortcut(tr("CTRL+O"));
    openMapAction->setIcon(QIcon(ICON_FILEOPEN));
    connect(openMapAction,SIGNAL(triggered()),this,SLOT(openMap()));

    saveMapAction = new QAction(tr("&Save Map"),this);
    saveMapAction->setStatusTip(tr("Save Map"));
    saveMapAction->setShortcut(tr("CTRL+S"));
    saveMapAction->setIcon(QIcon(ICON_FILESAVE));
    saveMapAction->setEnabled(false);
    connect(saveMapAction,SIGNAL(triggered()),this,SLOT(saveMap()));

    saveAsMapAction = new QAction(tr("Save Map As..."),this);
    saveAsMapAction->setStatusTip(tr("Save Map As..."));
    saveAsMapAction->setIcon(QIcon(ICON_FILESAVEAS));
    saveAsMapAction->setEnabled(false);
    connect(saveAsMapAction,SIGNAL(triggered()),this,SLOT(saveAsMap()));

    exportMapAction = new QAction(tr("Export Map As..."),this);
    exportMapAction->setStatusTip(tr("Export Map As..."));
    exportMapAction->setEnabled(false);
    connect(exportMapAction,SIGNAL(triggered()),this,SLOT(exportMap()));

    mergeMapsAction = new QAction(tr("&Merge Maps"),this);
    mergeMapsAction->setStatusTip(tr("Merge Maps"));
    mergeMapsAction->setShortcut(tr("CTRL+M"));
    mergeMapsAction->setIcon(QIcon(ICON_MERGE));
    connect(mergeMapsAction,SIGNAL(triggered()),this,SLOT(mergeMaps()));

    buildMapAction = new QAction(tr("&Build Map..."),this);
    buildMapAction->setStatusTip(tr("Build Map..."));
    buildMapAction->setShortcut(tr("CTRL+B"));
    buildMapAction->setIcon(QIcon(ICON_BUILD));
    connect(buildMapAction,SIGNAL(triggered()),this,SLOT(buildMap()));

    addImagesAction = new QAction(tr("&Add Images..."),this);
    addImagesAction->setStatusTip(tr("Add Images..."));
    addImagesAction->setShortcut(tr("CTRL+A"));
    addImagesAction->setIcon(QIcon(ICON_ADDVERTEX));
    addImagesAction->setEnabled(false);
    connect(addImagesAction,SIGNAL(triggered()),this,SLOT(addImages()));

    addEdgeAction = new QAction(tr("Add Edge..."),this);
    addEdgeAction->setStatusTip(tr("Add Edge..."));
    addEdgeAction->setIcon(QIcon(ICON_ADDEDGE));
    addEdgeAction->setEnabled(false);
    connect(addEdgeAction,SIGNAL(triggered()),this,SLOT(addEdge()));

    delVertexAction = new QAction(tr("Remove &Vertex..."),this);
    delVertexAction->setStatusTip(tr("Remove Vertex..."));
    delVertexAction->setEnabled(false);
    delVertexAction->setShortcut(tr("CTRL+V"));
    delVertexAction->setIcon(QIcon(ICON_DELVERTEX));
    connect(delVertexAction,SIGNAL(triggered()),this,SLOT(removeVertex()));

    delEdgeAction = new QAction(tr("Remove &Edge..."),this);
    delEdgeAction->setStatusTip(tr("Remove Edge..."));
    delEdgeAction->setEnabled(false);
    delEdgeAction->setShortcut(tr("CTRL+E"));
    delEdgeAction->setIcon(QIcon(ICON_DELEDGE));
    connect(delEdgeAction,SIGNAL(triggered()),this,SLOT(removeEdges()));

    delComponentAction = new QAction(tr("Remove &Connected Component..."),this);
    delComponentAction->setStatusTip(tr("Remove Connected Component..."));
    delComponentAction->setEnabled(false);
    delComponentAction->setShortcut(tr("CTRL+C"));
    delComponentAction->setIcon(QIcon(ICON_DELCOMP));
    connect(delComponentAction,SIGNAL(triggered()),this,SLOT(removeConnectedComp()));

    testAction = new QAction(tr("&Test"),this);
    testAction->setStatusTip(tr("Test function"));
    testAction->setShortcut(tr("CTRL+T"));
    testAction->setEnabled(true);
    connect(testAction,SIGNAL(triggered()),this,SLOT(test()));


    // IMAGE TOOLBAR
    openImageAction = new QAction(tr("&Open Image"),this);
    openImageAction->setStatusTip(tr("Open Image"));
    openImageAction->setShortcut(tr("CTRL+I"));
    openImageAction->setIcon(QIcon(ICON_FILEOPEN));
    connect(openImageAction,SIGNAL(triggered()),this,SLOT(openImage()));

    saveImageAction = new QAction(tr("Save Image"),this);
    saveImageAction->setStatusTip(tr("Save Image"));
    saveImageAction->setIcon(QIcon(ICON_FILESAVE));
    saveImageAction->setEnabled(false);
    connect(openImageAction,SIGNAL(triggered()),this,SLOT(saveImage()));

    saveAsImageAction = new QAction(tr("Save Image As..."),this);
    saveAsImageAction->setStatusTip(tr("Save Image As..."));
    saveAsImageAction->setIcon(QIcon(ICON_FILESAVEAS));
    saveAsImageAction->setEnabled(false);
    connect(openImageAction,SIGNAL(triggered()),this,SLOT(saveAsImage()));

    compareImageAction = new QAction(tr("&Compare Images"),this);
    compareImageAction->setStatusTip(tr("Compare Images"));
    compareImageAction->setShortcut(tr("CTRL+C"));
    compareImageAction->setEnabled(true);
    compareImageAction->setIcon(QIcon(ICON_IMAGECOMPARE));
    connect(compareImageAction,SIGNAL(triggered()),this,SLOT(compareImages()));

    // WINDOW TOOLBAR
    closeAllWindowsAction = new QAction(tr("Close All Windows"),this);
    closeAllWindowsAction->setStatusTip(tr("Close All Windows"));
    closeAllWindowsAction->setEnabled(true);
    connect(closeAllWindowsAction,SIGNAL(triggered()),this,SLOT(closeAllWindows()));

    closeAllImageWindowsAction = new QAction(tr("Close All Images"),this);
    closeAllImageWindowsAction->setStatusTip(tr("Close All Images"));
    closeAllImageWindowsAction->setEnabled(true);
    connect(closeAllImageWindowsAction,SIGNAL(triggered()),this,SLOT(closeAllImageWindows()));

    tileWindowsAction = new QAction(tr("Tile Windows"),this);
    tileWindowsAction->setStatusTip(tr("Tile Windows"));
    tileWindowsAction->setEnabled(true);
    connect(tileWindowsAction,SIGNAL(triggered()),this,SLOT(tileWindows()));

    // ABOUT TOOLBAR
    aboutAction = new QAction(tr("&About VisApp"),this);
    aboutAction->setStatusTip(tr("About VisApp"));
    aboutAction->setIcon(QIcon(ICON_INFO));
    connect(aboutAction,SIGNAL(triggered()),this,SLOT(about()));

    aboutQtAction = new QAction(tr("About &Qt"),this);
    aboutQtAction->setStatusTip(tr("About Qt"));
    connect(aboutQtAction,SIGNAL(triggered()),qApp,SLOT(aboutQt()));
}


void MainWindow::updateActions() {
    // Disable all actions
    saveMapAction->setEnabled(false);
    saveAsMapAction->setEnabled(false);
    addImagesAction->setEnabled(false);
    addEdgeAction->setEnabled(false);
    delVertexAction->setEnabled(false);
    delEdgeAction->setEnabled(false);
    delComponentAction->setEnabled(false);
    exportMapAction->setEnabled(false);

    saveImageAction->setEnabled(false);
    saveAsImageAction->setEnabled(false);

    if(mdiArea->currentSubWindow()) {
        if(qobject_cast<MapWidget *>(mdiArea->currentSubWindow()->widget())) {
            // Enable map actions
            saveMapAction->setEnabled(true);
            saveAsMapAction->setEnabled(true);
            addImagesAction->setEnabled(true);
            addEdgeAction->setEnabled(true);
            delVertexAction->setEnabled(true);
            delEdgeAction->setEnabled(true);
            delComponentAction->setEnabled(true);
            exportMapAction->setEnabled(true);
        }
        else if(qobject_cast<ImageWidget *>(mdiArea->currentSubWindow()->widget())) {
            // Enable image actions
            saveImageAction->setEnabled(true);
            saveAsImageAction->setEnabled(true);
        }
    }
}


void MainWindow::createMenus() {
    fileMenu = menuBar()->addMenu(tr("&File"));
    fileMenu->addAction(exitAction);    

    menuBar()->addSeparator();

    mapMenu = menuBar()->addMenu(tr("&Map"));
    mapMenu->addAction(openMapAction);
    mapMenu->addAction(saveMapAction);
    mapMenu->addAction(saveAsMapAction);
    mapMenu->addAction(exportMapAction);
    mapMenu->addSeparator();
    mapMenu->addAction(buildMapAction);
    mapMenu->addAction(addImagesAction);
    mapMenu->addAction(delVertexAction);
    mapMenu->addAction(addEdgeAction);
    mapMenu->addAction(delEdgeAction);
    mapMenu->addAction(delComponentAction);
    mapMenu->addSeparator();
    mapMenu->addAction(mergeMapsAction);
    mapMenu->addSeparator();
    mapMenu->addAction(testAction);

    menuBar()->addSeparator();

    imageMenu = menuBar()->addMenu(tr("&Image"));
    imageMenu->addAction(openImageAction);
    imageMenu->addAction(saveImageAction);
    imageMenu->addAction(saveAsImageAction);
    imageMenu->addSeparator();
    imageMenu->addAction(compareImageAction);

    menuBar()->addSeparator();

    windowMenu = menuBar()->addMenu(tr("&Window"));
    windowMenu->addAction(closeAllWindowsAction);
    windowMenu->addAction(closeAllImageWindowsAction);
    windowMenu->addAction(tileWindowsAction);

    menuBar()->addSeparator();

    helpMenu = menuBar()->addMenu(tr("&Help"));
    helpMenu->addAction(aboutAction);
    helpMenu->addAction(aboutQtAction);
}


void MainWindow::createToolBars() {
    fileToolBar = addToolBar(tr("File Toolbar"));
    fileToolBar->setMovable(false);
    fileToolBar->addAction(exitAction);
    fileToolBar->addSeparator();

    mapToolBar = addToolBar(tr("Map Toolbar"));
    mapToolBar->setMovable(false);
    QLabel* mLabel = new QLabel;
    mLabel->setText("Map:");
    mapToolBar->addWidget(mLabel);
    mapToolBar->addAction(openMapAction);
    mapToolBar->addAction(saveMapAction);
    mapToolBar->addAction(saveAsMapAction);
    mapToolBar->addAction(buildMapAction);
    mapToolBar->addAction(addImagesAction);
    mapToolBar->addAction(delVertexAction);
    mapToolBar->addAction(addEdgeAction);
    mapToolBar->addAction(delEdgeAction);
    mapToolBar->addAction(delComponentAction);
    mapToolBar->addAction(mergeMapsAction);
    mapToolBar->addSeparator();

    imageToolBar = addToolBar(tr("Image Toolbar"));
    imageToolBar->setMovable(false);
    QLabel* iLabel = new QLabel;
    iLabel->setText("Image:");
    mapToolBar->addWidget(iLabel);
    imageToolBar->addAction(openImageAction);
    imageToolBar->addAction(saveImageAction);
    imageToolBar->addAction(saveAsImageAction);
    imageToolBar->addAction(compareImageAction);

    aboutToolBar = addToolBar(tr("About Toolbar"));
    aboutToolBar->setMovable(false);
    aboutToolBar->addAction(aboutAction);
    //    addToolBarBreak();
}


void MainWindow::createStatusBar() {
    statusBar()->showMessage("Welcome to VisApp");
}

void MainWindow::about() {
    QMessageBox::about(this, tr("About VisApp..."),
                       tr("<h2>VisApp 1.0</h2>"
                          "<p>by Gorkem Erinc<br/>gerinc@ucmerced.edu</p>"
                          "<p>VisApp is a small application that helps you to "
                          "visualize, synthesize, and analyze appearance graphs. "
                          "It also provides some tools to merge appearance graphs, "
                          "analyze images with respect to their epipolar geometry "
                          "properties etc.</p>"));
}


void MainWindow::closeAllWindows() {
    while(mdiArea->subWindowList().size()>0)
    {
        QMdiSubWindow *sub = mdiArea->subWindowList()[0];
        mdiArea->removeSubWindow(sub);
        sub->close();
    }
}

void MainWindow::closeAllImageWindows() {
    QList<QMdiSubWindow*> list = mdiArea->subWindowList();
    QMdiSubWindow* win;
    foreach(win, list) {
        if(qobject_cast<ImageWidget *>(win->widget())) {
            win->close();
        }
    }
}

void MainWindow::tileWindows() {
    mdiArea->tileSubWindows();
}




void MainWindow::disableAllInputs()
{
    fileMenu->setEnabled(false);
    mapMenu->setEnabled(false);
    imageMenu->setEnabled(false);
    windowMenu->setEnabled(false);
    helpMenu->setEnabled(false);

    fileToolBar->setEnabled(false);
    aboutToolBar->setEnabled(false);
    mapToolBar->setEnabled(false);
    imageToolBar->setEnabled(false);
}

void MainWindow::enableAllInputs()
{
    fileMenu->setEnabled(true);
    mapMenu->setEnabled(true);
    imageMenu->setEnabled(true);
    windowMenu->setEnabled(true);
    helpMenu->setEnabled(true);

    fileToolBar->setEnabled(true);
    aboutToolBar->setEnabled(true);
    mapToolBar->setEnabled(true);
    imageToolBar->setEnabled(true);
}








/***************************/
/* IMAGE RELATED FUNCTIONS */
/***************************/

void MainWindow::openImage() {
    QStringList filenames = QFileDialog::getOpenFileNames(this,
                                                          tr("Open image"),".",
                                                          tr("All image files (*.ppm *.pgm)\n"
                                                             "PNG images (*.png)\n"
                                                             "JPG images (*.jpg)\n"
                                                             "PPM images (*.ppm)\n"
                                                             "PGM images (*.pgm)"));

    for(int i=0;i<filenames.size(); i++) {
        QString s = filenames[i];
        QFileInfo fi(s);

        ImageWidget* imageWidget = new ImageWidget();
        if(imageWidget->openImage(s)) {
            QMdiSubWindow *subwin = mdiArea->addSubWindow(imageWidget);
            subwin->setWindowIcon(QIcon(ICON_IMAGE));
            subwin->setWindowTitle(fi.fileName());
            imageWidget->show();
        }
    }
    if(filenames.size() > 0)
        statusBar()->showMessage("Images opened",2000);
}

void MainWindow::saveImage() {
    statusBar()->showMessage("Image saved",2000);
}

void MainWindow::saveAsImage() {
    statusBar()->showMessage("Image saved",2000);
}

void MainWindow::compareImages() {
    ImageComparisonDialog mydialog;
    if(mydialog.exec()) {
        QString s1 = mydialog.imageFile1;
        QString s2 = mydialog.imageFile2;
        Image *img1 = new Image();
        Image *img2 = new Image();
        img1->loadImage(s1.toStdString(),Image::PPM);
        img2->loadImage(s2.toStdString(),Image::PPM);
        img1->extractFeatures();
        img2->extractFeatures();

        QFileInfo fi1(s1);
        QFileInfo fi2(s2);

        ImageComparisonWidget* compImage = new ImageComparisonWidget(img1,img2);
        string wintitle = "Comparison: "+fi1.baseName().toStdString() +" vs "+fi2.baseName().toStdString();
        compImage->setWindowTitle(wintitle.c_str());
        if(mdiArea!=NULL) {
            QMdiSubWindow *s = mdiArea->addSubWindow(compImage);
            compImage->setSubWindowPtr(s);
        }
        compImage->show();
    }
}






/*************************/
/* MAP RELATED FUNCTIONS */
/*************************/

void MainWindow::openMap() {
    QString filename = QFileDialog::getOpenFileName(this, "Open map",".","Map files (*.map)");
    if(!filename.isEmpty()) {
        MapWidget* mapWidget = new MapWidget(filename);
        mapWidget->setMdiPtr(mdiArea);
        mapWidget->loadMapImage();
        QMdiSubWindow *subwin = mdiArea->addSubWindow(mapWidget);
        subwin->setWindowIcon(QIcon(ICON_MAP));
        subwin->setWindowTitle(mapWidget->getTitle());
        subwin->resize(mapWidget->size()+QSize(10,30));
        mapWidget->show();
        statusBar()->showMessage("Map opened",2000);
    }
}

void MainWindow::saveMap() {
    MapWidget *map = qobject_cast<MapWidget *>(mdiArea->currentSubWindow()->widget());
    QString filename = map->getFilename();
    if(filename.isEmpty()) {
        filename = QFileDialog::getSaveFileName(this, "Save Map",".","Map files (*.map)\nAll files (*.*)");
        if (filename.isEmpty())
            return;
        if(!filename.endsWith(".map"))
            filename = filename + ".map";
    }
    (qobject_cast<MapWidget *>(mdiArea->currentSubWindow()->widget()))->saveMap(filename);
    statusBar()->showMessage("Map saved",2000);
}

void MainWindow::saveAsMap() {
    QString filename = QFileDialog::getSaveFileName(this, "Save Map As...",".","Map files (*.map)\nAll files (*.*)");
    if(!filename.endsWith(".map"))
        filename = filename + ".map";
    (qobject_cast<MapWidget *>(mdiArea->currentSubWindow()->widget()))->saveMap(filename);
    statusBar()->showMessage("Map saved",2000);
}

void MainWindow::exportMap() {
    //QString fullpath = QFileDialog::getSaveFileName(this, "Save Map As...",".","Output files (*.png *.gif *.jpg *.jpeg *.pdf *.eps *.ps)\nPNG (*.png)\nGIF (*.gif)\nJPG (*.jpg)\nJPEG (*.jpeg)\nPDF (*.pdf)\nEPS (*.eps)\nPS (*.ps)");
    QString filename = QFileDialog::getSaveFileName(this, "Export Map As...",".","PNG image (*.png)\nAll files (*.*)");
    if(!filename.endsWith(".png"))
        filename = filename + ".png";
    (qobject_cast<MapWidget *>(mdiArea->currentSubWindow()->widget()))->exportMap(filename);
    statusBar()->showMessage("Map exported",2000);
}




void MainWindow::mergeMaps() {
    MapMergeDialog mywindow;

    if(mywindow.exec()) {
        QStringList files = mywindow.getSelectedFiles();
        int method = mywindow.getSelectedMethod();
        int sc = mywindow.getSelectedStopCriteria();
        int st = mywindow.getSelectedStyle();
        double val = mywindow.getStopValue();
        int iter = mywindow.getIter();
        bool isSave = mywindow.isSave();

        if(files.size()>0) {
            // merge them and show the result
            MapWidget* mapWidget = new MapWidget;
            mapWidget->mergeMaps(files,(SimMap::MergeMethod::Type)method, (SimMap::MergeStyle::Type)st, (SimMap::MergeStopCriteria::Type)sc, val, iter, isSave);
            mapWidget->setMdiPtr(mdiArea);
            mapWidget->loadMapImage();
            QMdiSubWindow *subwin = mdiArea->addSubWindow(mapWidget);
            subwin->setWindowIcon(QIcon(ICON_MAP));
            subwin->setWindowTitle(mapWidget->getTitle());
            subwin->resize(mapWidget->size()+QSize(10,30));
            mapWidget->show();
            statusBar()->showMessage("Map opened",2000);
        }
        /*
        QStringList files = mywindow.getSelectedFiles();
        MapWidget* mapWidget = new MapWidget;
        mapWidget->setMdiPtr(mdiArea);
        QString filename = files.at(0);
        if(mapWidget->openMap(filename)) {
            files.removeAt(0);
            mapWidget->mergeMaps(files,method);
            mapWidget->loadMapImage();
            mdiArea->addSubWindow(mapWidget);
            mdiArea->currentSubWindow()->setWindowIcon(QIcon(ICON_MAP));
            mdiArea->currentSubWindow()->setWindowTitle(mapWidget->getTitle());
            mdiArea->currentSubWindow()->resize(mapWidget->size()+QSize(10,30));
            mapWidget->show();
            statusBar()->showMessage("Map opened",2000);
        }
*/
    }
}


void MainWindow::buildMap() {
    AddImagesDialog mywindow(QString("Building map..."),QString("&Build Map"));

    if(mywindow.exec()) {
        QStringList files = mywindow.getSelectedFiles();

        /* TEMPCODE
        QStringList files;
        for(int i=0; i<files1.count(); i=i+2) {
            files.push_front(files1.pop_back());
        }
        */


        bool isForced = mywindow.isForced();
        if(files.count() > 0) {
            MapWidget* mapWidget = new MapWidget;
            mapWidget->setMdiPtr(mdiArea);
            connect(mapWidget,SIGNAL(Busy()),this,SLOT(disableAllInputs()));
            connect(mapWidget,SIGNAL(notBusy()),this,SLOT(enableAllInputs()));

            QMdiSubWindow *subwin = mdiArea->addSubWindow(mapWidget);

            struct timeval td_start,td_end;

            gettimeofday(&td_start,NULL);
            mapWidget->buildMap(files, isForced);
            gettimeofday(&td_end,NULL);
            double elapsed = 1000000.0 * (td_end.tv_sec -td_start.tv_sec);
            elapsed += (td_end.tv_usec - td_start.tv_usec);
            cout<<"TIME: "<<(double)(elapsed/1000000.0)<<endl;

            //cout<<"TIME: "<<(double)final/((double)CLOCKS_PER_SEC)<<endl;

            mapWidget->setMaximumSize(mdiArea->size());
            mapWidget->loadMapImage();
            subwin->setWindowIcon(QIcon(ICON_MAP));
            subwin->setWindowTitle(mapWidget->getTitle());
            subwin->resize(mapWidget->size()+QSize(10,30));
            mapWidget->show();
            statusBar()->showMessage("Map created",2000);
        }
    }
}

void MainWindow::addImages() {
    AddImagesDialog mywindow(QString("Adding images to the map..."),QString("&Add to Map"));
    if(mywindow.exec()) {
        QStringList files = mywindow.getSelectedFiles();
        bool isForced = mywindow.isForced();
        bool addEdges = mywindow.addEdges();
        if(files.count()) {
            MapWidget *mapWidget = qobject_cast<MapWidget *>(mdiArea->currentSubWindow()->widget());
            mapWidget->addImages(files, isForced, addEdges);
            mapWidget->loadMapImage();
            mdiArea->currentSubWindow()->resize(mapWidget->size()+QSize(10,30));
            mapWidget->show();
            statusBar()->showMessage("Images added to the map",2000);
        }
    }
}

void MainWindow::addEdge() {

    EdgeInput *ein = new EdgeInput();
    ein->show();
    if(ein->exec()) {
        int v1 = ein->getVertex1Id();
        int v2 = ein->getVertex2Id();
        if(v1>0 && v2>0) {
            MapWidget *mapWidget = qobject_cast<MapWidget *>(mdiArea->currentSubWindow()->widget());
            if(mapWidget->addEdge(v1,v2)) {
                mapWidget->loadMapImage();
                mdiArea->currentSubWindow()->resize(mapWidget->size()+QSize(10,30));
                mapWidget->show();
                statusBar()->showMessage("Edge added to the map",2000);
            }
            else {
                statusBar()->showMessage("Error: Edge could not be added to the map",2000);
            }
        }
    }
}


void MainWindow::removeVertex() {
    VertexInput *vin = new VertexInput();
    vin->show();
    if(vin->exec()) {
        MapWidget *mapWidget = qobject_cast<MapWidget *>(mdiArea->currentSubWindow()->widget());
        int v = vin->getVertexId();
        if(v>-1 && mapWidget->removeVertex(v)) {
            mapWidget->loadMapImage();
            mdiArea->currentSubWindow()->resize(mapWidget->size()+QSize(10,30));
            mapWidget->show();
            statusBar()->showMessage("Vertex deleted from the map",2000);
        }
        else {
            statusBar()->showMessage("Error: Vertex could not be deleted from the map",2000);
        }
    }
}


void MainWindow::removeConnectedComp() {
    VertexInput *vin = new VertexInput();
    vin->show();
    if(vin->exec()) {
        MapWidget *mapWidget = qobject_cast<MapWidget *>(mdiArea->currentSubWindow()->widget());
        int c = vin->getVertexId();
        if(c>-1 && mapWidget->removeConnectedComp(c)) {
            mapWidget->loadMapImage();
            mdiArea->currentSubWindow()->resize(mapWidget->size()+QSize(10,30));
            mapWidget->show();
            statusBar()->showMessage("ConnectedComp deleted from the map",2000);
        }
        else {
            statusBar()->showMessage("Error: ConnectedComp could not be deleted from the map",2000);
        }
    }
}



void MainWindow::removeEdges() {
    EdgeInputText *ein = new EdgeInputText();
    ein->show();
    if(ein->exec()) {
        MapWidget *mapWidget = qobject_cast<MapWidget *>(mdiArea->currentSubWindow()->widget());
        if(mapWidget->removeEdges(ein->limit1(),ein->limit2())) {
            mapWidget->loadMapImage();
            mdiArea->currentSubWindow()->resize(mapWidget->size()+QSize(10,30));
            mapWidget->show();
            statusBar()->showMessage("Edge deleted from the map",2000);
        }
        else {
            statusBar()->showMessage("Error: Edge could not be deleted from the map",2000);
        }
    }
}

void MainWindow::removeEdge() {
    EdgeInput *ein = new EdgeInput();
    ein->show();
    if(ein->exec()) {
        int v1 = ein->getVertex1Id();
        int v2 = ein->getVertex2Id();
        if(v1>0 && v2>0) {
            MapWidget *mapWidget = qobject_cast<MapWidget *>(mdiArea->currentSubWindow()->widget());
            if(mapWidget->removeEdge(v1,v2)) {
                mapWidget->loadMapImage();
                mdiArea->currentSubWindow()->resize(mapWidget->size()+QSize(10,30));
                mapWidget->show();
                statusBar()->showMessage("Edge deleted from the map",2000);
            }
            else {
                statusBar()->showMessage("Error: Edge could not be deleted from the map",2000);
            }
        }
    }
}

void MainWindow::test() {
    MapWidget *mapWidget = (qobject_cast<MapWidget *>(mdiArea->currentSubWindow()->widget()));
    mapWidget->test();
    /*
    MapWidget* mapWidget = new MapWidget();
    mapWidget->setMdiPtr(mdiArea);
    mapWidget->test();
    mapWidget->loadMapImage();
    QMdiSubWindow *subwin = mdiArea->addSubWindow(mapWidget);
    subwin->setWindowIcon(QIcon(ICON_MAP));
    subwin->setWindowTitle(mapWidget->getTitle());
    subwin->resize(mapWidget->size()+QSize(10,30));
    mapWidget->show();
    statusBar()->showMessage("Test function results...",2000);
    */
}
