#include "mapmergedialog.h"
#include <QtGui>
#include <iostream>

using namespace std;

MapMergeDialog::MapMergeDialog(QWidget *parent) :
        QDialog(parent)
{
    files.clear();
    setWindowTitle("Merging maps...");

    QLabel* descriptionLabel = new QLabel("Create a list of the maps you would like to merge");

    // BUTTONS
    addButton = new QPushButton("&Add Map",this);
    addButton->setAutoDefault(false);
    removeButton = new QPushButton("&Remove Map",this);
    removeButton->setAutoDefault(false);
    removeAllButton = new QPushButton("Remove All",this);
    removeAllButton->setAutoDefault(false);
    mergeButton = new QPushButton("&Merge",this);
    //mergeButton->setAutoDefault(false);

    upButton = new QPushButton("&Up",this);
    upButton->setAutoDefault(false);
    downButton = new QPushButton("&Down",this);
    downButton->setAutoDefault(false);

    mapList = new QListWidget(this);
    mapList->setSelectionMode(QAbstractItemView::SingleSelection);
    mapList->setMinimumWidth(580);

    QVBoxLayout *vbox = new QVBoxLayout;
    vbox->addWidget(addButton);
    vbox->addWidget(removeButton);
    vbox->addWidget(removeAllButton);
    vbox->addSpacing(15);
    vbox->addWidget(upButton);
    vbox->addWidget(downButton);
    vbox->addStretch(1);

    QHBoxLayout *hbox = new QHBoxLayout;
    hbox->addWidget(mapList);
    hbox->addSpacing(10);
    hbox->addLayout(vbox);

    // METHOD AND STOPPING CRITERIA
    QLabel* methodLabel = new QLabel("Method:");
    methodList = new QComboBox;
    methodList->addItem("Brute Force (uniform)");
    methodList->addItem("Brute Force (degreeMin)");
    methodList->addItem("Quick Connect");
    methodList->addItem("Brute Force (order)");
    //methodList->addItem("Uniform (degree)");
    //methodList->addItem("Uniform (feats)");
    //methodList->addItem("Markov (edge)");
    //methodList->addItem("Markov (weight)");
    //methodList->addItem("Markov (weightReverse)");
    //methodList->addItem("Markov (clusterIn)");
    //methodList->addItem("Markov (clusterOut)");
    //methodList->addItem("Smart Method-1");
    //methodList->addItem("Smart Method-2");
    //methodList->addItem("Smart Method-3");
    //methodList->addItem("Smart Method-4");
    //methodList->addItem("Hu-Newton");
    //methodList->addItem("Merge Test Function");
    methodList->setCurrentIndex(2);

    QLabel* styleLabel = new QLabel("Style:");
    styleList = new QComboBox;
    styleList->addItem("Simultaneous");
    styleList->addItem("Pairwise");

    QLabel* stopCriteriaLabel = new QLabel("StopCriteria:");
    stopCriteriaList = new QComboBox;
    stopCriteriaList->addItem("None");
    stopCriteriaList->addItem("Time");
    stopCriteriaList->addItem("Percentage");

    stopValue = new QLineEdit("0.0");
    QDoubleValidator *dvalidator = new QDoubleValidator();
    stopValue->setValidator(dvalidator);
    stopValue->setMaximumSize(100,40);
    stopValue->setMaxLength(10);
    stopValue->setCursorPosition(0);
    stopValue->setSelection(0,10);
    stopValue->setVisible(false);

    QHBoxLayout *hmethodbox = new QHBoxLayout;
    hmethodbox->addWidget(methodLabel);
    hmethodbox->addWidget(methodList);
    hmethodbox->addSpacing(5);
    hmethodbox->addWidget(styleLabel);
    hmethodbox->addWidget(styleList);
    hmethodbox->addSpacing(5);
    hmethodbox->addWidget(stopCriteriaLabel);
    hmethodbox->addWidget(stopCriteriaList);
    hmethodbox->addSpacing(3);
    hmethodbox->addWidget(stopValue);
    hmethodbox->addStretch(1);

    // DETAILS
    QLabel* iterLabel = new QLabel("Number of iterations:");
    iterValue = new QLineEdit("1");
    QIntValidator *ivalidator = new QIntValidator();
    iterValue->setValidator(ivalidator);
    iterValue->setMaximumSize(50,40);
    iterValue->setMaxLength(5);
    iterValue->setCursorPosition(0);
    iterValue->setSelection(0,10);
    iterValue->setVisible(true);

    saveCheckBox = new QCheckBox("Record Process");
    saveCheckBox->setChecked(false);

    QHBoxLayout *hdetailsbox = new QHBoxLayout;
    hdetailsbox->addWidget(iterLabel);
    hdetailsbox->addWidget(iterValue);
    hdetailsbox->addSpacing(50);
    hdetailsbox->addWidget(saveCheckBox);
    hdetailsbox->addStretch(1);


    // MAIN LAYOUT
    QVBoxLayout *mainLayout = new QVBoxLayout(this);
    mainLayout->addWidget(descriptionLabel);
    mainLayout->addLayout(hbox);
    mainLayout->addSpacing(10);
    mainLayout->addLayout(hmethodbox);
    mainLayout->addSpacing(10);
    mainLayout->addLayout(hdetailsbox);
    mainLayout->addSpacing(20);
    mainLayout->addWidget(mergeButton,0,Qt::AlignCenter);

    setLayout(mainLayout);

    updateButtons();

    connect(addButton, SIGNAL(clicked()), this, SLOT(add()));
    connect(removeButton, SIGNAL(clicked()), this, SLOT(remove()));
    connect(removeAllButton, SIGNAL(clicked()), this, SLOT(removeAll()));
    //connect(mergeButton, SIGNAL(pressed()), this, SLOT(accept()));
    connect(mergeButton, SIGNAL(clicked()), this, SLOT(accept()));
    connect(this,SIGNAL(accepted()),this,SLOT(merge()));
    connect(mapList, SIGNAL(itemSelectionChanged()), this, SLOT(updateButtons()));
    connect(stopCriteriaList, SIGNAL(currentIndexChanged(int)), this, SLOT(updateButtons()));
    connect(upButton, SIGNAL(clicked()), this, SLOT(moveUp()));
    connect(downButton, SIGNAL(clicked()), this, SLOT(moveDown()));

    openFilesPath = QDir::currentPath() + "/maps";
}

void MapMergeDialog::updateButtons()
{
    if(mapList->currentRow()>-1)
        removeButton->setEnabled(true);
    else
        removeButton->setEnabled(false);

    if(mapList->count()>0)
        removeAllButton->setEnabled(true);
    else
        removeAllButton->setEnabled(false);

    if(mapList->count()>1) {
        mergeButton->setEnabled(true);
        mergeButton->setFocus();
    }
    else
        mergeButton->setEnabled(false);

    if(stopCriteriaList->currentIndex() > 0) {
        stopValue->setFocus();
        stopValue->setVisible(true);
    }
    else
        stopValue->setVisible(false);


    if(mapList->count() > 1 && mapList->selectedItems().count() > 0 && mapList->row(mapList->selectedItems().front()) > 0) {
        upButton->setEnabled(true);
    }
    else
        upButton->setEnabled(false);

    if(mapList->count() > 1 && mapList->selectedItems().count() > 0 && mapList->row(mapList->selectedItems().front()) < mapList->count()-1) {
        downButton->setEnabled(true);
    }
    else
        downButton->setEnabled(false);
}

void MapMergeDialog::moveUp() {
    int r = mapList->row(mapList->selectedItems().front());
    QListWidgetItem * it = mapList->takeItem(r);
    mapList->insertItem(r-1,it);
    mapList->setCurrentRow(r-1);
}

void MapMergeDialog::moveDown() {
    int r = mapList->row(mapList->selectedItems().front());
    QListWidgetItem * it = mapList->takeItem(r);
    mapList->insertItem(r+1,it);
    mapList->setCurrentRow(r+1);
}

void MapMergeDialog::add() {
    QFileDialog::Options options;
    options |= QFileDialog::DontUseNativeDialog;
    QString selectedFilter;
    QStringList files = QFileDialog::getOpenFileNames(
            this, "Select maps to merge",
            openFilesPath,
            "Map Files (*.map)",
            &selectedFilter,
            options);
    if (files.count()) {
        //cout<<"p:"<<openFilesPath.toStdString()<<endl;
        //openFileNamesLabel->setText(QString("[%1]").arg(files.join(", ")));
        foreach(QString s, files) {
            QFileInfo fi(s);
            openFilesPath = fi.absolutePath();
            QList<QListWidgetItem*> found = mapList->findItems(s,Qt::MatchExactly);
            if(found.count()==0) {
                mapList->addItem(s);
            }
        }
        updateButtons();
    }
}

void MapMergeDialog::remove() {
    mapList->takeItem(mapList->currentRow());
    updateButtons();
}

void MapMergeDialog::removeAll() {
    mapList->clear();
    updateButtons();
}

void MapMergeDialog::merge() {
    for(int i=0; i<mapList->count(); i++) {
        QListWidgetItem* file = mapList->item(i);
        files.push_back(file->text());
    }
}

double MapMergeDialog::getStopValue() {
    QString str = stopValue->text();
    double val = 0.0;
    bool ok = false;
    val = str.toDouble(&ok);
    if(ok)
        return val;
    else
        return 0.0;
}


int MapMergeDialog::getIter() {
    QString str = iterValue->text();
    int val = 0.0;
    bool ok = false;
    val = str.toInt(&ok);
    if(ok)
        return val;
    else
        return 1;
}
