#include "simmap.h"
#include "defs.h"
#include <stdio.h>
#include <stdlib.h>
#include <QFileInfo>
#include <QStringList>
#include <QString>
#include <QDir>
#include <QRegExp>
#include "configuration.h"
#include <string>

#include <boost/graph/graphviz.hpp>

// For clustering
#include <boost/graph/bc_clustering.hpp>
#include <boost/graph/iteration_macros.hpp>
#include <boost/graph/connected_components.hpp>

#include <boost/graph/johnson_all_pairs_shortest.hpp>
#include <boost/graph/dijkstra_shortest_paths.hpp>

#include <queue>
//#include <ctime>
#include <time.h>

using namespace converter;
using namespace iu;
using namespace std;


ofstream outTest;
ofstream aaa;

mt19937 SimMap::randengine;
double SimMap::mergeStartTime = 0;
MergeLog SimMap::mergeLog;


SimMap::SimMap()
{
    mapId = 0;

    randengine.seed(time(NULL));

    //    outTest.open("test1.txt");
    maxIndex = -1;

    // Check if sys folder exists..
    QDir dir;
    if(!dir.exists("sys"))
        dir.mkdir("sys");

    // Create a temp folder in sys
    QString dirpath("sys/map");
    int count = 0;
    while(true) {
        count++;
        dirpath = "sys/map" + QString::number(count);

        if(!dir.exists(dirpath)) {
            dir.mkdir(dirpath);
            break;
        }
    }
    dataDir = dirpath.toStdString()+"/";

    dict = new Dictionary(appConf.CLUSTER_FILE, appConf.INDEX_FILE);
    sg = new SimGraph();
    sg->setDataDir(dataDir);

    filepath = "";
}

// will call readMap
SimMap::SimMap(path fname)
{
    mapId = 0;

    randengine.seed(time(NULL));

    //    outTest.open("test1.txt");
    maxIndex = -1;

    // Check if sys folder exists..
    QDir dir;
    if(!dir.exists("sys"))
        dir.mkdir("sys");

    // Create a temp folder in sys
    QString dirpath("sys/map");
    int count = 0;
    while(true) {
        count++;
        dirpath = "sys/map" + QString::number(count);

        if(!dir.exists(dirpath)) {
            dir.mkdir(dirpath);
            break;
        }
    }
    dataDir = dirpath.toStdString()+"/";

    dict = NULL;
    sg = NULL;
    readMap(fname);
}







SimMap::~SimMap()
{
    //    outTest.close();

    cout<<"simmap destructor"<<endl;
    delete dict;
    delete sg;
    cleanImageList();
    char systemcall[200];
    sprintf(systemcall,"rm -rf %s",dataDir.c_str());
    cout<<"called: "<<systemcall<<endl;
    ::system(systemcall);
}



bool SimMap::isEmpty() {
    if(sg->numVertices() == 0 && imageLookup.size() == 0)
        return true;
    return false;
}

void SimMap::cleanImageList() {
    cout<<"clear imagelist"<<endl;
    BOOST_FOREACH(ImageTable::value_type &it, imageLookup) {
        delete it.second;
    }
    imageLookup.clear();
}














//void SimMap::test(QStringList files) {
void SimMap::test(QString imagefile1,QString imagefile2) {

    cout<<"TEST"<<endl;

    //QString imagefile1("/home/gorkem/code/data/test/imgComp/19vs23_1/RGB_00323.ppm");
    //QString imagefile2("/home/gorkem/code/data/test/imgComp/19vs23_1/RGB_00938.ppm");
    //QString imagefile1("/home/gorkem/code/data/test/imgComp/26vs28_1/RGB_00045.ppm");
    //QString imagefile2("/home/gorkem/code/data/test/imgComp/26vs28_1/RGB_00340.ppm");
    //QString imagefile1=files[0];
    //QString imagefile2=files[1];

    Image *img1=new Image();
    Image *img2=new Image();
    img1->loadImage(imagefile1.toStdString(),Image::PPM);
    img2->loadImage(imagefile2.toStdString(),Image::PPM);
    img1->extractFeatures();
    img2->extractFeatures();

    vertexDescriptor v1 = addImageAsVertex(img1);
    //vertexDescriptor v2 = addImageAsVertex(img2);

    ZointMatchMap matches;
    int maxvote = getVotes(img2, matches);
    CandidateList candidates;


    struct timeval tStart,tEnd;
    gettimeofday(&tStart,NULL);
    enforceGeometricConstraints(matches, candidates, true);
    gettimeofday(&tEnd,NULL);
    double elapsed = (tEnd.tv_sec -tStart.tv_sec) + (tEnd.tv_usec - tStart.tv_usec)/1000000.0;

    cout<<maxvote<<endl;
    cout<<candidates.size()<<endl;
    cout<<"time: "<<elapsed<<endl;

    /*
    ImageTable::iterator it;
    for(it=imageLookup.begin(); it!=imageLookup.end(); it++) {
        cout<<"\n\nImage: "<<it->first<<endl;
        it->second->printWords();
    }
    */

    /*
    int data[] = {2,4,5,23,0,-1,49};
    Mat hop(1,7,CV_32S,data);
    Mat sorted;
    sortIdx(hop,sorted,CV_SORT_DESCENDING);
    cout<<hop<<endl;
    cout<<sorted<<endl;
    exit(0);


    cout<<"COUNTING:"<<endl;
    VertexTable::iterator it;
    for(it=getGraph()->getVertexLookup()->begin(); it!=getGraph()->getVertexLookup()->end(); it++)
    {
        cout<<it->first<<endl;
    }

    VertexTable::iterator it2 = getGraph()->getVertexLookup()->find(26);
    size_t index = std::distance(getGraph()->getVertexLookup()->begin(), it2);
    cout<<"INDEX: "<<index<<endl;


    cout<<endl<<endl;

    pair<vertexIterator,vertexIterator> vrange = vertices(getGraph()->g);
    for(vertexIterator v=vrange.first; v!=vrange.second; v++) {
        cout<<"vertex: "<<getGraph()->g[*v].index<<endl;
    }

    cout<<endl<<endl;

    for(int i=0; i<getGraph()->numVertices(); i++) {
        cout<<"numbered: "<<getGraph()->g[vertex(i,getGraph()->g)].index<<endl;
    }

    cout<<endl<<endl;
    //cout<<getGraph()->g
*/

    /*
    vector<int> clusters;
    int num = sg->computeClusters(clusters);
    cout<<num<<endl;
    */
}

void SimMap::test2(SimMap *m1, SimMap *m2)
{
    /*
    int startIndex;
    Mat A = computeProbMat(m1,m2,SimMap::SMART_1, startIndex);
    cout<<A<<endl;
    */
}


int SimMap::chooseIndex(int i)
{
    int idx;

    // Choose a good index for the image
    if(i > maxIndex) {
        // index not in use
        maxIndex = i;
        idx = i;
    }
    else
        idx = ++maxIndex;

    /*
    if(i<0) {
        // no index assigned
        idx = ++maxIndex;
    }
    else if(i > maxIndex) {
        // index not in use
        maxIndex = i;
        idx = i;
    }
    else { // within the index range
        if(imageLookup.find(i) == imageLookup.end()) {
            // unallocated index: use that index...
            idx = i;
        }
        else {
            // that index is currently in use
            idx = ++maxIndex;
        }
    }
    */

    Debug(1)<<"Chosen index: "<<idx<<endl;
    return idx;
}



int SimMap::getVotes(Image *img, ZointMatchMap &matches) {
    int maxvote = -1;
    int localizedimage = -1;

    dict->findMatches(img, matches); // will be filled with PointMatches like <img1,img2>
    Debug(2)<<"total matches: "<<matches.size()<<endl;

    // Get the image with maximum votes
    BOOST_FOREACH(ZointMatchMap::value_type &mm, matches) {
        ZointMatchList *ml = &(mm.second);
        if((int)(ml->first.size()) > maxvote) {
            maxvote = (int)(ml->first.size());
            localizedimage = mm.first;
        }
    }

    Debug(2)<<"Max vote for image("<<localizedimage<<"): "<<maxvote<<endl;

    return maxvote;
}


bool SimMap::enforceGeometricConstraints(ZointMatchMap &matches, CandidateList &candidates, bool forceSimilarImages)
{
    candidates.clear();

    bool isTooSimilar = false;

    // Check for lower bound
    BOOST_FOREACH(ZointMatchMap::value_type &m, matches) {
        //printf("Processing image %d: # %d matches\n",m.first, m.second.size());

        int candimgIndex = m.first;
        ZointMatchList *pml = &m.second;

        //ZointMatchList *pml = &m.second; //pml points to pair<pointList,pointList>
        if((int)pml->first.size() >= appConf.MINMATCHES) {

            Debug(1)<<"Candidate image match "<<candimgIndex<<" with "<<pml->first.size()<<" matches"<<endl;

            // Given two sets of properties
            // compute F and count number of supporting matches by enforcing minimal error
            // this will return the most probable F and the number of supporters for this F
            CvMat* F = cvCreateMat(3,3,CV_64FC1);
            cv::Mat t(F);

#if _TIMING==1
            struct timeval tStart,tEnd;
            gettimeofday(&tStart,NULL);
#endif

            int numofsupporters = enforceFundMatConstraint(t, *pml, 0b001);
#if _TIMING==1
            gettimeofday(&tEnd,NULL);
            double elapsed = (tEnd.tv_sec -tStart.tv_sec) + (tEnd.tv_usec - tStart.tv_usec)/1000000.0;
            Debug(0)<<"Fundamental matrix computation time: "<<elapsed<<endl;
            timingFund.push_back(elapsed);
#endif

            //int numofsupporters = pml->first.size();
            // if more than limit, quit process since there is AT LEAST one vertex too similar to this image
            if(numofsupporters > appConf.MAXMATCHES)
                isTooSimilar = true;

            if(isTooSimilar && forceSimilarImages==false) {
                Debug(0)<<"Too similar so quit looking"<<endl;
                return isTooSimilar;
            }
            else if(numofsupporters >= appConf.MINMATCHES) {
                ZointMatchList *inliers = new ZointMatchList;
                for(unsigned int k=0; k<pml->first.size(); k++) {
                    inliers->first.push_back(pml->first[k]);
                    inliers->second.push_back(pml->second[k]);
                }
                candidates[make_pair(numofsupporters,candimgIndex)] = make_pair(F,inliers);

                Debug(1)<<"passed with: "<<numofsupporters<<endl;
                //cout<<" PASSED"<<endl;
            }
            else {
                Debug(1)<<"failed with: "<<numofsupporters<<endl;
                cvReleaseMat(&F);
                //cout<<" FAILED"<<endl;
            }
        }
    }
    return isTooSimilar;
}




vector<int> SimMap::addImage(QString imagefile, int indx, bool force, int trackingMode, bool addEdge)
{
#if _TIMING==1
    struct timeval tStart,tEnd;
    double elapsed;
    gettimeofday(&tStart,NULL);
#endif
    Image *img = new Image();

    if(indx > -1) {
        img->setIndex(indx);
    }
    else { // try to extract index info from filename
        QFileInfo fi(imagefile);
        QRegExp reg("_0*(\\d+)\\.");
        reg.setPatternSyntax(QRegExp::RegExp2);
        if(reg.indexIn(fi.fileName()) != -1) {
            QString num = reg.cap(1);
            img->setIndex(num.toInt());
        }
    }
#if _TIMING==1
    gettimeofday(&tEnd,NULL);
    elapsed = (tEnd.tv_sec -tStart.tv_sec) + (tEnd.tv_usec - tStart.tv_usec)/1000000.0;
    timingImgIndex.push_back(elapsed);
#endif
    Debug(3)<<"\n\nAttempting to add image: "<<imagefile.toStdString()<<endl;



    if(imagefile.endsWith(".pgm",Qt::CaseInsensitive)) {
        img->loadImage(imagefile.toStdString(),Image::PGM);
        return addImage(img,force,trackingMode,addEdge);
    }
    else if(imagefile.endsWith(".ppm",Qt::CaseInsensitive)) {
#if _TIMING==1
        gettimeofday(&tStart,NULL);
#endif
        img->loadImage(imagefile.toStdString(),Image::PPM);
#if _TIMING==1
        gettimeofday(&tEnd,NULL);
        elapsed = (tEnd.tv_sec -tStart.tv_sec) + (tEnd.tv_usec - tStart.tv_usec)/1000000.0;
        timingLoad.push_back(elapsed);

        gettimeofday(&tStart,NULL);
#endif
        img->extractFeatures();
#if _TIMING==1
        gettimeofday(&tEnd,NULL);
        elapsed = (tEnd.tv_sec -tStart.tv_sec) + (tEnd.tv_usec - tStart.tv_usec)/1000000.0;
        timingSift.push_back(elapsed);
#endif

        return addImage(img,force,trackingMode,addEdge);
    }
    else {
        delete img;
        img = NULL;
        vector<int> temp;
        return temp;
    }
}


vertexDescriptor SimMap::addImageAsVertex(Image *img, bool keepData) {
    int oldIndex = img->getIndex();
    int chosenIndex = chooseIndex(img->getIndex());
    img->setIndex(chosenIndex);

    Debug(1)<<"index set to: "<<img->getIndex()<<" from: "<<oldIndex<<endl;

    // add the image to the list
    imageLookup[img->getIndex()] = img;

    // Create a vertex with that image
    vertexDescriptor v = sg->addVertex(img);
    getGraph()->g[v].oldindex = oldIndex;

    //sg->updateStats();

    // Add the location of the features from the image to the database
    // no ties with corresponding image pointer
    dict->addImage(img);

    if(!keepData)
        img->deleteData();

    return v;
}


vector<int> SimMap::addImage(Image *img, bool force, int trackingMode, bool addEdge) {

    Debug(2)<<"Processing image: "<<img->getFilename()<<endl;

    // Extract its features if not extracted before
    if(img->features.size()==0) {
        //cout<<"features extracted"<<endl;
        img->extractFeatures();
    }

    if(img->features.size()==0) {
        delete img;
        img = NULL;
        vector<int> temp;
        return temp;
    }

#if _TIMING==1
    gettimeofday(&time_start,NULL);
#endif

    vector<int> newIndices;

    // Find similarities between query image and all other images in the DB
    ZointMatchMap matches;

#if _TIMING==1
    struct timeval tStart,tEnd;
    gettimeofday(&tStart,NULL);
#endif
    int maxvote = getVotes(img, matches);
#if _TIMING==1
    gettimeofday(&tEnd,NULL);
    timingDict.push_back((tEnd.tv_sec -tStart.tv_sec) + (tEnd.tv_usec - tStart.tv_usec)/1000000.0);
#endif

    CandidateList candidates;

    if(force) {
        enforceGeometricConstraints(matches, candidates, true);

        vertexDescriptor v = addImageAsVertex(img);
        if(addEdge)
            sg->addEdges(v, candidates, imageLookup);
        newIndices.push_back(getGraph()->g[v].index);

#if _TIMING==1
        gettimeofday(&time_end,NULL);
        double elapsed = (time_end.tv_sec -time_start.tv_sec) + (time_end.tv_usec - time_start.tv_usec)/1000000.0;
        timingAdd.push_back(elapsed);
        //cout<<"IMAGE ADDITION TIME: "<<elapsed<<endl;
#endif

        return newIndices;
    }




    bool backtracking = false;

    bool tooSimilar = enforceGeometricConstraints(matches, candidates, true);
    bool notEnough = (candidates.size()==0);

    if(notEnough) {
        Debug(0)<<"NOT ENOUGH - Start BT"<<endl;
        backtracking = true;
    }
    else {
        bool isconnected = getGraph()->isConnected(candidates);

        if(tooSimilar) {
            Debug(0)<<"TOO SIMILAR"<<endl;
            if(isconnected) { // has last cluster edge
                if(getGraph()->improvesGraph(candidates)) {
                    Debug(0)<<"CONNECTED - ADD"<<endl;
                    Debug(0)<<"Have edges"<<endl;
                    vertexDescriptor v = addImageAsVertex(img);
                    if(addEdge)
                        getGraph()->addEdges(v, candidates, imageLookup);
                    newIndices.push_back(getGraph()->g[v].index);

                    for(unsigned int i=0; i<trackingList.size(); i++) {
                        delete trackingList[i];
                    }
                    trackingList.clear();
                }
                else {
                    Debug(0)<<"CONNECTED - IGNORE"<<endl;
                    trackingList.push_back(img);
                }
            }
            else {
                Debug(0)<<"NOT CONNECTED - Start BT"<<endl;
                backtracking = true;
            }
        }
        else { // similar enough
            Debug(0)<<"SIMILAR ENOUGH"<<endl;
            if(isconnected) { // has last cluster edge
                Debug(0)<<"CONNECTED - ADD"<<endl;
                Debug(0)<<"Have edges"<<endl;
                vertexDescriptor v = addImageAsVertex(img);
                if(addEdge)
                    getGraph()->addEdges(v, candidates, imageLookup);
                newIndices.push_back(getGraph()->g[v].index);

                for(unsigned int i=0; i<trackingList.size(); i++) {
                    delete trackingList[i];
                }
                trackingList.clear();
            }
            else {
                Debug(0)<<"NOT CONNECTED - Start BT"<<endl;
                backtracking = true;
            }
        }
    }





    if(backtracking) {
        Debug(0)<<"STARTING BT"<<endl;
        if(trackingList.size()==0) {
            Debug(0)<<"EMPTY TRACKINGLIST"<<endl;
            vertexDescriptor v = addImageAsVertex(img);
            newIndices.push_back(getGraph()->g[v].index);
        }
        else {
            vector<Image*> processList;
            processList.push_back(img);

            while(trackingList.size()>0) {
                Image* myimage = trackingList.back();
                trackingList.pop_back();

                cout<<"backtracking process image: "<<myimage->getFilename()<<endl;

                // Find similarities between query image and all other images in the DB
                ZointMatchMap matches;
                getVotes(myimage, matches);

                CandidateList candidates;
                enforceGeometricConstraints(matches, candidates,true);

                bool isconnected = getGraph()->isConnected(candidates);
                if(isconnected)
                    Debug(0)<<"ISCONNECTED"<<endl;
                else
                    Debug(0)<<"NOT CONNECTED"<<endl;

                if (candidates.size() > 0 && isconnected) {   // at least one edge creation needed
                    Debug(0)<<"ADDING VERTEX"<<endl;
                    vertexDescriptor v = addImageAsVertex(myimage);
                    if(addEdge)
                        sg->addEdges(v, candidates, imageLookup);
                    newIndices.push_back(getGraph()->g[v].index);

                    for(unsigned int i=0; i<trackingList.size(); i++) {
                        delete trackingList[i];
                    }
                    trackingList.clear();
                }
                else { // no edges
                    Debug(0)<<"NO EDGES"<<endl;
                    if(trackingList.size()==0) {
                        Debug(0)<<"FAILED BACKTRACKING"<<endl;
                        // failed backtracking
                        vertexDescriptor v = addImageAsVertex(myimage);
                        newIndices.push_back(getGraph()->g[v].index);
                    }
                    else {
                        processList.push_back(myimage);
                    }
                }
            }

            for(unsigned int i=0; i<processList.size(); i++) {
                vector<int> ilist = addImage(processList[i]);
                for(unsigned int j=0; j<ilist.size(); j++) {
                    newIndices.push_back(ilist[j]);
                }
            }
        }
    } // if(backtracking)


#if _TIMING==1
    gettimeofday(&time_end,NULL);
    double elapsed = (time_end.tv_sec -time_start.tv_sec) + (time_end.tv_usec - time_start.tv_usec)/1000000.0;
    timingAdd.push_back(elapsed);
    //cout<<"IMAGE ADDITION TIME: "<<elapsed<<endl;
#endif

    return newIndices;
}





// Given an image localize that image in the map
// and return a pointer to the localized image
// ATT: Here we can define the location of the image as a prop distribution over all images
// since votes is actually a histogram
int SimMap::localizeImage(Image *img) {
    int imgIndex = img->getIndex();

    ZointMatchMap matches;
    dict->findMatches(img,matches);

    int maxvote = -1;
    int localizedImage = -1;

    BOOST_FOREACH(ZointMatchMap::value_type &m, matches) {
        if((int)m.second.first.size() > appConf.MINMATCHES) {

            int candimgIndex = m.first;

            printf("Candidate image match (%d-%d) : # %d matches\n",imgIndex, candimgIndex, m.second.first.size());

            // Given two sets of properties
            // compute F and count number of supporting matches by enforcing minimal error
            // this will return the most probable F and the number of supporters for this F
            CvMat* F = cvCreateMat(3,3,CV_64FC1);
            ZointMatchList *pl = &m.second;
            cv::Mat t(F);
            int numofsupporters = enforceFundMatConstraint(t, *pl);

            if(numofsupporters >= appConf.MINMATCHES) {
                if(maxvote < numofsupporters) {
                    maxvote = numofsupporters;
                    localizedImage = candimgIndex;
                }
            }
        }
    }

    // Currently we say the image is localized if it has more than "minVotes" votes
    if(localizedImage != -1) {
        printf("Image(%d) is localized to image(%d) with %d number of votes",img->getIndex(),localizedImage,maxvote);
    }
    else {
        printf("Image(%d) could not be localized.",img->getIndex());
    }
    return localizedImage;
}


bool SimMap::addEdge(int v1, int v2) {
    edgeDescr e = getGraph()->addEdge(v1,v2);
    return e.second;
}

bool SimMap::removeVertex(int vIndex) {

    cout<<"deleting vertex: "<<vIndex<<endl;
    if(getGraph()->vertexLookup.count(vIndex)) {
        // delete from imagelookup
        Vertex* v = getGraph()->getVertex(vIndex);
        imageLookup.erase(v->image->getIndex());

        for(int i=0; i<v->image->words.size(); i++) {
            getDictionary()->invIndex[v->image->words[i].first].erase(v->image->getIndex());
        }

        return getGraph()->removeVertex(vIndex);
    }
    return false;
}

bool SimMap::removeEdge(int v1, int v2) {
    return getGraph()->removeEdge(v1,v2);
}

bool SimMap::removeEdges(vector<pair<int,int> > lim1, vector<pair<int,int> > lim2) {
    return getGraph()->removeEdges(lim1,lim2);
}

bool SimMap::removeConnectedComp(int c) {
    bool ret = false;

    getGraph()->computeConnectedComponents();

    typedef map<int,vector<int> > temptype;
    BOOST_FOREACH(temptype::value_type &t, getGraph()->components) {
        if(t.second.size() <= c) {
            // delete all vertices in this component
            for(int i=0; i<t.second.size(); i++) {
                int vIndex = t.second[i];
                cout<<"removing clusterVertex: "<<vIndex<<endl;
                removeVertex(vIndex);
                ret = true;
            }
        }
    }
    if(ret)
        getGraph()->computeConnectedComponents();
    getGraph()->updateStats();
    return ret;

}



bool SimMap::writeMap(path fpath, bool writeIndvfiles) {

    cout<<"Writing map into file ("<<fpath.string()<<")..."<<endl;

    // Create a temp folder in sys
    QString dirpath("sys/temp");
    QDir dir;
    int count = 0;
    while(true) {
        count++;
        dirpath = "sys/temp" + QString::number(count);

        if(!dir.exists(dirpath)) {
            dir.mkdir(dirpath);
            break;
        }
    }
    string tempfoldername = dirpath.toStdString() + "/";

    string filename = fpath.stem();

    /*
    BOOST_FOREACH(ImageTable::value_type &it, imageLookup) {
        if(!saveImageToFile(it.second,tempfoldername)) {
            cout<<"ERROR: Image "<<it.second->getIndex()<<" could not be saved!"<<endl;
        }
    }
*/

    string graphFilename = filename+".sg";
    string imagelistFilename = filename+".il";
    string dictFilename = filename+".di";
    string configFilename = "map.conf";

    dict->writeDictionary(tempfoldername+dictFilename);
    writeImageList(tempfoldername+imagelistFilename);
    sg->writeGraph(tempfoldername+graphFilename);

    // Write config file
    Debug(1)<<"Writing configuration to file: "<<tempfoldername+configFilename<<endl;

    ofstream conf((tempfoldername+configFilename).c_str());
    if(conf.is_open()) {
        conf << "files: " << endl;
        conf << "graphFilename = " << graphFilename << endl;
        conf << "dictFilename = " << dictFilename << endl;
        conf << "imagelistFilename = " << imagelistFilename << endl;
        conf << endl;
        conf << "configurations: " << endl;
        conf << "FMATSAMPLES = " << appConf.FMATSAMPLES << endl;
        conf << "FMATMININLIERS = " << appConf.FMATMININLIERS << endl;
        conf << "MAXMATCHES = " << appConf.MAXMATCHES << endl;
        conf << "MINMATCHES = " << appConf.MINMATCHES << endl;
        conf << "FEATUREDIM = " << appConf.FEATUREDIM << endl;
        conf << "CLUSTER_FILE = " << appConf.CLUSTER_FILE << endl;
        conf << "INDEX_FILE = " << appConf.INDEX_FILE << endl;
        conf << "TRACKINGMODE = " << appConf.TRACKINGMODE << endl;
        conf.close();
    }
    else {
        cout<<"ERROR: Configuration file ("<<configFilename<<") could not be opened to write"<<endl;
        return false;
    }

    char systemcall[200];

    // Compress these files to create a map file
    sprintf(systemcall,"cd %s && tar -cf %s * && cd ../..",tempfoldername.c_str(),fpath.string().c_str());
    ::system(systemcall);


    if(writeIndvfiles) {
        sprintf(systemcall,"mv %s%s .",tempfoldername.c_str(),graphFilename.c_str());
        ::system(systemcall);
        sprintf(systemcall,"mv %s%s .",tempfoldername.c_str(),imagelistFilename.c_str());
        ::system(systemcall);
        sprintf(systemcall,"mv %s%s .",tempfoldername.c_str(),dictFilename.c_str());
        ::system(systemcall);
        sprintf(systemcall,"mv %s%s .",tempfoldername.c_str(),configFilename.c_str());
        ::system(systemcall);
    }

    sprintf(systemcall,"rm -rf %s",tempfoldername.c_str());
    ::system(systemcall);

    setFilepath(fpath);

    return true;
}


bool SimMap::readMap(path fpath)
{
    //boost::filesystem::path filepath(fp.toStdString());
    cout<<"Reading map from file ("<<fpath.string()<<")..."<<endl;

    // First deallocate current data in the map
    if(sg!=NULL)
        delete sg;
    if(dict!=NULL)
        delete dict;
    cleanImageList();
    char systemcall[200];
    sprintf(systemcall,"rm -f %s*",dataDir.c_str());
    ::system(systemcall);


    /*
    QFileInfo fi(filepath);
    QString fullfname = fi.fileName();
    QStringList flist = fullfname.split(".");
    string filename;
    if(flist.size()>0) {
        filename = flist.at(0).toStdString();
    }
    else
        filename = fullfname.toStdString();
*/

    // Extract these files to read the map file
    sprintf(systemcall,"tar -C %s -xf %s",dataDir.c_str(), fpath.string().c_str());
    ::system(systemcall);

    string configFilename = "map.conf";
    string graphFilename, imagelistFilename, dictFilename;

    // Read config file
#if DEBUG > -1
    cout<<"Reading configuration to file: "<<dataDir+configFilename<<endl;
#endif
    ifstream conf((dataDir+configFilename).c_str());

    // To be able to old map files where configuration file is saved as filename.conf instead of map.conf
    // This is changed because in the case of map filename change, it was not possible to open the map
    // since it was looking for a newfilename.conf where the file was saved as filename.conf
    if(!conf.is_open()) {
        string filename = fpath.stem();
        configFilename = filename+".conf";
        conf.open((dataDir+configFilename).c_str());
    }

    if(conf.is_open()) {
        istringstream parse;
        string line, temp, valuestr;
        int value;
        while(getline(conf,line)) {
            parse.clear();
            parse.str(line);
            parse >> temp;

            if(temp=="graphFilename") {
                parse >> temp >> graphFilename;
            }
            else if(temp=="dictFilename") {
                parse >> temp >> dictFilename;
            }
            else if(temp=="imagelistFilename") {
                parse >> temp >> imagelistFilename;
            }
            else if(temp=="FMATSAMPLES") {
                parse >> temp >> value;
                appConf.FMATSAMPLES = value;
            }
            else if(temp=="FMATMININLIERS") {
                parse >> temp >> value;
                appConf.FMATMININLIERS = value;
            }
            else if(temp=="MAXMATCHES") {
                parse >> temp >> value;
                appConf.MAXMATCHES = value;
            }
            else if(temp=="MINMATCHES") {
                parse >> temp >> value;
                appConf.MINMATCHES = value;
            }
            else if(temp=="FEATUREDIM") {
                parse >> temp >> value;
                appConf.FEATUREDIM = value;
            }
            else if(temp=="CLUSTER_FILE") {
                parse >> temp >> valuestr;
                appConf.CLUSTER_FILE = valuestr;
            }
            else if(temp=="INDEX_FILE") {
                parse >> temp >> valuestr;
                appConf.INDEX_FILE = valuestr;
            }
            else if(temp=="TRACKINGMODE") {
                parse >> temp >> value;
                appConf.TRACKINGMODE = value;
            }
            else {
                continue;
            }
        }
        conf.close();
    }
    else {
        cout<<"ERROR: Configuration file ("<<configFilename<<") could not be opened to read"<<endl;
        return false;
    }

    dict = new Dictionary(appConf.CLUSTER_FILE,appConf.INDEX_FILE);
    sg = new SimGraph();
    sg->setDataDir(dataDir);

    readImageList(dataDir+imagelistFilename);
    dict->readDictionary(dataDir+dictFilename,imageLookup);
    sg->readGraph(dataDir+graphFilename,imageLookup);
    updateStats();



    // Check if map files contains all image files
    bool filecheck = true;
    BOOST_FOREACH(ImageTable::value_type &it, imageLookup) {
        Image * img = it.second;
        if(img->getFilename() == "") {
            filecheck = false;
            break;
        }
        else {
            //QFile f((dataDir+img->getFilename()).c_str());
            QFile f(img->getFilename().c_str());
            if (!f.exists()) {
                filecheck = false;
                break;
            }
        }
    }

    if (filecheck) {
        /*
        // load image data from file
        BOOST_FOREACH(ImageTable::value_type &it, imageLookup) {
            Image * img = it.second;
            string imgfilename = dataDir + img->getFilename();
            img->loadImage(imgfilename,img->getFileFormat());

            // Update filename to the temp local folder
            QFileInfo fi(imgfilename.c_str());
            QString absfilepath = fi.absoluteFilePath();
            img->setFilename(absfilepath.toStdString());
            //cout<<"index: "<<img->getIndex()<<endl;
        }
        */
    }
    else {
        cout<<"ERROR: Images cannot be loaded from the map file because the map file does not contain all images"<<endl;
        return false;
    }

    trackingList.clear();

    setFilepath(fpath);

    cout<<"Done"<<endl;
    return true;
}


bool SimMap::writeImageList(string filename) {
    Debug(2)<<"Writing imageList to file: "<<filename<<endl;

    ofstream out(filename.c_str());
    if(!out.is_open())
        return false;
    BOOST_FOREACH(ImageTable::value_type &it, imageLookup) {
        out << "ImageIndex=" << it.second->getIndex() << ", ";
        //cout<<"write filename: "<<it.second->getFilename()<<endl;
        out << "Filename=" << it.second->getFilename() << ", ";
        out << "Fileformat=" << it.second->getFileFormat() << ", ";
        out << "Timestamp=" << it.second->getTimestamp().tv_sec << " " <<it.second->getTimestamp().tv_usec << ", ";
        out << "Location=" << it.second->getPose().x << " " << it.second->getPose().y << " " << it.second->getPose().z;
        /*
        out << "Words=";
        for(unsigned int i=0; i<it.second->words.size(); i++) {
            out<<it.second->words[i]<<" ";
        }
        */
        out << endl;
    }
    out.close();
    return true;
}

bool SimMap::readImageList(string filename) {
    Debug(2)<<"Reading imageList from file: "<<filename<<endl;

    ifstream in(filename.c_str());
    if(!in.is_open())
        return false;
    string line;
    while(getline(in,line)) {
        if(!converter::isEmpty(line)) {
            size_t found1, found2;
            found1 = line.find("=");
            found2 = line.find(",",found1+1);
            string index(line,found1+1, found2-found1-1);
            int imageIndex = converter::str2int(index);

            found1 = line.find("=",found2+1);
            found2 = line.find(",",found1+1);
            string filename(line,found1+1, found2-found1-1);

            found1 = line.find("=",found2+1);
            found2 = line.find(",",found1+1);
            string fileformat(line,found1+1, found2-found1-1);

            found1 = line.find("=",found2+1);
            found2 = line.find(",",found1+1);
            string mytime(line,found1+1, found2-found1-1);

            found1 = line.find("=",found2+1);
            found2 = line.find(",",found1+1);
            string myloc(line,found1+1, found2-found1-1);

            found1 = line.find("=",found2+1);
            string words = line.substr(found1+1);

            Image* img = new Image();
            img->setIndex(imageIndex);
            if(imageIndex>maxIndex)
                maxIndex = imageIndex;

            if(!converter::isEmpty(filename))
                img->setFilename(filename);
            //cout<<"read filename: "<<filename<<endl;

            if(!converter::isEmpty(fileformat))
                img->setFileFormat(str2int(fileformat));
            if(!converter::isEmpty(mytime)) {
                stringstream parse(mytime);
                long int sec,usec;
                parse >> sec >> usec;
                struct timeval tv;
                tv.tv_sec = sec;
                tv.tv_usec = usec;
                img->setTimestamp(tv);
            }
            if(!converter::isEmpty(myloc)) {
                stringstream parse(myloc);
                double x, y, theta;
                parse >> x >> y >> theta;
                img->setPose(Pose(x,y,theta));
            }
            /*
            if(!converter::isEmpty(words)) {
                stringstream parse(words);
                int idx;
                while(parse >> idx) {
                    img->words.push_back(idx);
                }
            }
            */
            imageLookup[imageIndex] = img;
            img->deleteData();
        }
    }
    in.close();
    return true;
}


bool SimMap::saveImageToFile(Image *img, string path) {

    if(img->isEmpty()) {
        if(img->loadImageFromFile()==-1) {
            return false;
        }
    }

    char imgfilename[50];
    if(img->getImageFormat()==Image::RGB) {
        sprintf(imgfilename,"RGB_%05d.ppm",img->getIndex());
        img->saveImage(path+imgfilename, Image::PPM);
        img->setFilename(imgfilename);
        return true;
    }
    else if(img->getImageFormat()==Image::GRAY) {
        sprintf(imgfilename,"GRAY_%05d.pgm",img->getIndex());
        img->saveImage(path+imgfilename, Image::PGM);
        img->setFilename(imgfilename);
        return true;
    }
    return false;
}

void SimMap::printGraph(QString &imagefile, QString imageformat, QString graphlayout) {
    QDir dir(dataDir.c_str());
    if(imagefile.isEmpty()) {
        imagefile = dir.absolutePath()+"/"+graphlayout+"."+imageformat;
    }
    QString graphfile = dir.absolutePath()+"/"+graphlayout+".graph";
    sg->writeGraph(graphfile.toStdString());
    if(!sg->printGraph(graphfile.toStdString(), imagefile.toStdString(),imageformat.toStdString(),graphlayout.toStdString()))
        filepath = "";
    else
        sg->computeGraphLayout(graphfile.toStdString(), graphlayout.toStdString());
}


void SimMap::writeStats(QString filepath) {
    sg->writeStats(filepath.toStdString());
}

void SimMap::showTiming() {
    cout<<"TIMING RESULTS: "<<endl;

    double sum;
    int size;

    sum=0.0;
    size = timingImgIndex.size();
    for(int i=0; i<size; i++)
        sum+=timingImgIndex[i];
    cout<<"Total img load time: "<<sum<<" over "<<size<<" calls (avg: "<<sum/size<<")"<<endl;

    sum=0.0;
    size = timingLoad.size();
    for(int i=0; i<size; i++)
        sum+=timingLoad[i];
    cout<<"Total img load time: "<<sum<<" over "<<size<<" calls (avg: "<<sum/size<<")"<<endl;

    sum=0.0;
    size = timingSift.size();
    for(int i=0; i<size; i++)
        sum+=timingSift[i];
    cout<<"Total sift time: "<<sum<<" over "<<size<<" calls (avg: "<<sum/size<<")"<<endl;

    sum = 0.0;
    size = timingDict.size();
    for(int i=0; i<size; i++)
        sum+=timingDict[i];
    cout<<"Total dict time: "<<sum<<" over "<<size<<" calls (avg: "<<sum/size<<")"<<endl;

    sum=0.0;
    size = timingFund.size();
    for(int i=0; i<size; i++)
        sum+=timingFund[i];
    cout<<"Total fund time: "<<sum<<" over "<<size<<" calls (avg: "<<sum/size<<")"<<endl;

    sum=0.0;
    size = timingAdd.size();
    for(int i=0; i<size; i++)
        sum+=timingAdd[i];
    cout<<"Total img add: "<<sum<<" over "<<size<<" calls (avg: "<<sum/size<<")"<<endl;
}




double SimMap::scoreMap()
{
    /*
    Graph gnew= getGraph()->g;

    double inf = 10000;
    double score = 0.0;

    int count = 0;
    pair<vertexIterator,vertexIterator> vrange = vertices(gnew);
    vertexIterator v1,v2;
    for(v1=vrange.first; v1!=vrange.second; v1++) {
        for(v2=v1+1; v2!=vrange.second; v2++) {
            count++;

            cv::Point2d p1 = gnew[*v1].gridmapPose.center;
            cv::Point2d p2 = gnew[*v2].gridmapPose.center;

            double cost = 0.0;

            vertexDescriptor start = *v1;
            vertexDescriptor goal = *v2;
            vector<vertexDescriptor> mypath;
            AStar mysearch;
            if(mysearch.searchThroughGraph(gnew,start,goal,mypath)) {
                //printf("Path bulundu\n");

                vector<vertexDescriptor>::iterator vv1 = mypath.begin();
                vector<vertexDescriptor>::iterator vv2 = vv1+1;
                while(vv1!=mypath.end() && vv2!=mypath.end()) {
                    //cout<<"edge: "<<sg->g[*vv1].index<<" -> "<<sg->g[*vv2].index<<" : ";
                    cv::Point2d p1t = gnew[*vv1].gridmapPose.center;
                    cv::Point2d p2t = gnew[*vv2].gridmapPose.center;

                    double dist = norm(Mat(p1t),Mat(p2t));

                    cost += dist;

                    vv1++;
                    vv2++;
                }
            }
            else {
                cout<<"*************************BULUNAMADI*********************"<<endl;
                cost += inf;
            }

            double dist = norm(Mat(p1),Mat(p2));
            double distEst = cost;

            //cout<<"before score: "<<distEst<<" "<<dist<<endl;

            score += fabs((distEst-dist));///distEst);
        }
    }
    return score/(double)count;
*/

    //Graph gnew= getGraph()->g;
    /*
    ofstream out;

    out.open("adj.txt");
    pair<vertexIterator,vertexIterator> vrange = vertices(getGraph()->g);
    for(vertexIterator v1=vrange.first; v1!=vrange.second; v1++) {
        for(vertexIterator v2=vrange.first; v2!=vrange.second; v2++) {
            int i1 = getGraph()->g[*v1].index;
            int i2 = getGraph()->g[*v2].index;
            if(i1!=i2) {
                pair<edgeDescriptor,bool> edgebool = edge(*v1,*v2, getGraph()->g);
                if(edgebool.second)
                    out<<getGraph()->gEdge[edgebool.first].cost<<" ";
                else
                    out<<"0 ";
            }            else
                out<<"0 ";
        }
        out<<endl;
    }
    out.close();
*/



    /*
    const int n = num_vertices(getGraph()->g);
    int **D = new int*[n];
    for(int i=0;i<n;i++)
        D[i] = new int[n];

    std::vector<int> d(n, (std::numeric_limits<int>::max)());
    johnson_all_pairs_shortest_paths(getGraph()->g,D,distance_map(&d[0]));

    int c = 0;
    double dist = 0.0;
    pair<vertexIterator,vertexIterator> vrange = vertices(getGraph()->g);
    int i=0;
    for(vertexIterator v1=vrange.first; v1!=vrange.second; v1++) {
        int j=0;
        for(vertexIterator v2=vrange.first; v2!=vrange.second; v2++) {
            if(getGraph()->g[*v1].mapId != getGraph()->g[*v2].mapId) {
                c++;
                dist += D[i][j];
            }
            j++;
        }
        i++;
    }
    // counting everything twice
    c = c/2;
    dist = dist/2;

    dist = dist/(double)c;

    for(int i=0;i<n;i++) {
        delete D[i];
    }
    delete D;
    return dist;
*/

    Mat d = getGraph()->graphLaplacian();
    int n = d.rows;

    //ofstream out("laplacian.txt");
    //out<<d<<endl;
    //out.close();

    //timespec tic,toc;
    //clock_gettime(CLOCK_PROCESS_CPUTIME_ID,&tic);
    Mat E;
    eigen(d,E);
    //clock_gettime(CLOCK_PROCESS_CPUTIME_ID,&toc);
    //double t = toc.tv_sec -tic.tv_sec + (toc.tv_nsec-tic.tv_nsec)/1000000000.0;
    //cout<<t<<endl;


    double s = E.at<float>(n-2,0);

    if(n>1)
        return s;
    else
        return 0;
}















// oldnewIndex
void SimMap::unionMaps(SimMap* m, map<int,int> &oldnewIndex) {

    if(numVertices()>0)
        mapId++;

    oldnewIndex.clear();

    // transfer vertices
    pair<vertexIterator,vertexIterator> vrange = vertices(m->getGraph()->g);
    for(vertexIterator v=vrange.first; v!=vrange.second; v++) {
        Image *img = new Image(m->getGraph()->g[*v].image);
        vertexDescriptor vd = addImageAsVertex(img,true);
        getGraph()->g[vd].mapId = mapId; // assign new mapId for vertices of new map
        int oldIndex = getGraph()->g[vd].oldindex;
        int newIndex = getGraph()->g[vd].index;
        oldnewIndex[oldIndex] = newIndex;
        if(newIndex!=oldIndex) {
            char labelstr[100];
            //sprintf(labelstr,"%d[%s:%d]",newIndex,m->getFilename().c_str(),oldIndex);
            sprintf(labelstr,"%d[%d]",newIndex,oldIndex);
            getGraph()->g[vd].label = string(labelstr);
        }
    }

    // transfer edges
    pair<edgeIterator, edgeIterator> erange = edges(m->getGraph()->g);
    for(edgeIterator eit=erange.first; eit!=erange.second; eit++) {
        int idx1 = oldnewIndex[m->getGraph()->gEdge[*eit].v1];
        int idx2 = oldnewIndex[m->getGraph()->gEdge[*eit].v2];
        edgeDescr e = getGraph()->addEdge(idx1,idx2);
        if(e.second) {
            getGraph()->gEdge[e.first] = m->getGraph()->gEdge[*eit];
            getGraph()->gWeight[e.first] = 1;
            getGraph()->gEdge[e.first].v1 = idx1;
            getGraph()->gEdge[e.first].v2 = idx2;
            getGraph()->gEdge[e.first].mapId = mapId;
        }
    }
    updateStats();
}






Mat SimMap::computeProbMat(MergeMethod::Type method, int &startIndex) {

    Mat A(getGraph()->numVertices(),getGraph()->numVertices(),CV_64F, Scalar(0.0));
    startIndex = -1;

    if(method == MergeMethod::BRUTEORDER) {
        for(int i=0; i<A.rows-1; i++) {
            A.at<double>(i,i+1) = 1.0;
        }
        A.at<double>(A.rows-1,0) = 1.0;
        startIndex = 0;
    }
    else if(method == MergeMethod::BRUTEUNIFORM) {
        A = Scalar(1.0/A.cols);
    }
    else if(method == MergeMethod::BRUTEDEGREEMIN) {
        pair<vertexIterator,vertexIterator> vrange = vertices(getGraph()->g);
        int i=0;
        for(vertexIterator v=vrange.first; v!=vrange.second; v++) {
            A.col(i) = 1/(double)degree(*v,getGraph()->g); // it will be normalized later
            i++;
        }
    }

    if(startIndex < 0) // not assigned specifically
        startIndex = selectStartVertex(A);

    return A;
}


int SimMap::selectStartVertex(Mat &A) {
    //cout<<"----------------------- SELECT A NEW START VERTEX -----------------"<<endl;

    Mat row(1,A.cols,A.type(),Scalar(0.0));
    for(int i=0; i<A.rows; i++) {
        row += A.row(i);
    }
    return selectNextVertex(row);

    return 0;
}


// selects the next vertex probabilistically
int SimMap::selectNextVertex(Mat row) {

    if(row.cols == 0)
        return -1;
    else if(row.cols == 1)
        return 0;


    // normalize the probabilities
    double n = sum(row)[0];
    if(areSame(n,0.0))
        return -1; // nothing to choose
    row = row/n;

    Mat cum = row.clone();
    for(int i=1; i<cum.cols; i++) {
        cum.at<double>(0,i) += cum.at<double>(0,i-1);
    }
    double maxval = cum.at<double>(0,cum.cols-1);

    uniform_real<> distribution(0,maxval);
    variate_generator<boost::mt19937&,boost::uniform_real<> > myrandom(randengine,distribution);
    double val = myrandom();
    for(int i=0; i<cum.cols; i++) {
        if(cum.at<double>(0,i) >= val)
            return i;
    }
    return cum.cols-1;

    //  // CHOOSE MAX
    //    // compute what to get next
    //    double maxVal;
    //    cv::Point maxPt;
    //    minMaxLoc(row,0,&maxVal,0,&maxPt);
    //    return maxPt.x;
}


void SimMap::normalizeMat(Mat &A) {
    for(int i=0; i<A.rows; i++) {
        double n = sum(A.row(i))[0];
        if(!areSame(n,0.0)) {
            A.row(i) = A.row(i)/n;
        }
    }
}






SimMap* SimMap::mergeMaps(QStringList mapfiles, MergeMethod::Type method, MergeStyle::Type style, MergeStopCriteria::Type sc, double val, int MCiter, bool logging)
{
    string methodstr = "";
    if(method==MergeMethod::QUICKCONNECT)
        methodstr = "newCode_quickConnect";
    else if(method==MergeMethod::BRUTEUNIFORM)
        methodstr = "bruteUniform";
    else if(method==MergeMethod::BRUTEDEGREEMIN)
        methodstr = "bruteDegree";
    cout<<"Selected method: "<<methodstr<<endl;

    string stylestr = "";
    if(style==MergeStyle::PAIRWISE)
        stylestr = "pairwise";
    else if(style==MergeStyle::SIMULTANEOUS)
        stylestr = "simultaneous";
    cout<<"Selected style: "<<stylestr<<endl;

    QString filename = "";
    if (logging || true) {
        filename = "snapshots_%1_%2_%3_p001_i%4.txt";
        //filename = "edgeSim_%1_%2_%3_p001_i%4.txt";
        //filename = "edgeSim_%1_smart4Degree_algCon_%2.txt";

        QString mapstr = "";
        for(int j=0;j<mapfiles.size();j++) {
            size_t found1 = mapfiles[j].toStdString().find_last_of("/\\")+1+3; // +3 for /Run
            size_t found2 = mapfiles[j].toStdString().find_last_of(".map")-3; // -3 for .map
            mapstr = mapstr + mapfiles[j].toStdString().substr(found1,found2-found1).c_str() + "vs";
        }
        mapstr.chop(2);

        filename = filename.arg(mapstr).arg(methodstr.c_str()).arg(stylestr.c_str());
        //filename = filename.arg(mapstr);
    }

    cout<<"******** LOADING MAPS ********"<<endl;
    SimMap *maps[mapfiles.size()];
    for(int j=0;j<mapfiles.size();j++) {
        maps[j] = new SimMap(mapfiles[j].toStdString());
    }

    SimMap *m;
    for (int i=1; i<=MCiter; i++) {

        struct timeval tStart,tEnd;
        gettimeofday(&tStart,NULL);

        if(logging) {
            SimMap::mergeLog.reset();
            SimMap::mergeLog.filename = filename.arg(i, 3, 10, QChar('0')).toStdString();
        }
        string fname = filename.arg(i, 3, 10, QChar('0')).toStdString();
        //string fname = filename.arg(i, 1, 10, QChar('0')).toStdString();

        // for quickconnect
        SimMap::mergeLog.explorationScoreCaptionInterval = 30;//10;
        SimMap::mergeLog.refinementScoreCaptionInterval = 60;//100;
        // for bruteforce
        SimMap::mergeLog.timeInterval = 3; // here logTime is used because it is easier to take average this way...


        aaa.open("edgeSim.txt");

        if(style==MergeStyle::SIMULTANEOUS) {
            m = new SimMap();
            cout<<"******** UNION MAPS ********"<<endl;
            map<int,int> oldnewIndices[mapfiles.size()];
            for(int j=0;j<mapfiles.size();j++) {
                m->unionMaps(maps[j],oldnewIndices[j]);
            }

            cout<<"******** CONNECTING MAPS ********"<<endl;
            if(method==MergeMethod::QUICKCONNECT) {
                m->quickConnect_simultaneous(maps,oldnewIndices,mapfiles.size(),sc,val,logging);
            }
            else if(method==MergeMethod::BRUTEUNIFORM || method==MergeMethod::BRUTEDEGREEMIN) {
                m->randomConnect_simultaneous(maps, oldnewIndices, mapfiles.size(), method, sc, val, logging);
                //m->randomConnect(maps[0],maps[1],oldnewIndices[0],oldnewIndices[1], method, sc, val);
            }
        }
        else if(style==MergeStyle::PAIRWISE) {
            if(method==MergeMethod::QUICKCONNECT) {
                m = quickConnect_pairwise(maps,mapfiles.size(),sc,val,logging);
            }
            else if(method==MergeMethod::BRUTEUNIFORM || method==MergeMethod::BRUTEDEGREEMIN) {
                m = randomConnect_pairwise(maps,mapfiles.size(),method,sc,val,logging);
            }
        }

        gettimeofday(&tEnd,NULL);
        double elapsed = (tEnd.tv_sec -tStart.tv_sec) + (tEnd.tv_usec - tStart.tv_usec)/1000000.0;
        cout<<"MERGING TIME: "<<elapsed<<" seconds"<<endl;

        aaa.close();
        char systemcall[200];
        sprintf(systemcall,"mv edgeSim.txt edgeSim_%s",fname.c_str());
        cout<<systemcall<<endl;
        ::system(systemcall);

        if(i<MCiter-1)
            delete m;
    }

    // Clear memory
    for(int j=0;j<mapfiles.size();j++) {
        delete maps[j];
    }

    m->updateStats();
    return m;


    /* // Getting all possible permutations for pairwise merging
    int n=1;
    for(int i=mapfiles.size(); i>0; i--)
        n = n*i;

    pair<int,int> permutations[n];
    int c=0;
    for(int j=0; j<n; j++) {
        for(int k=0; k<n; k++) {
            if(j!=k) {
                permutations[c++] = make_pair(j,k);
            }
        }
    }
    int limit;

    //SimMap *m1 = maps[permutations[k].first];
    //SimMap *m2 = maps[permutations[k].second];
    //m->connectMaps(m1, m2, oldnewIndex[permutations[k].second], A, method, sc, val, startIndex);
    */
}





void SimMap::randomConnect(SimMap *m1, SimMap *m2, map<int,int> &oldnewIndex1, map<int,int> &oldnewIndex2, MergeMethod::Type method, MergeStopCriteria::Type sc, double val, LoggingCriteria::Type logging) {

    int sel = -1;
    Mat A = m2->computeProbMat(method,sel);


    bool done = false;
    int count = 0;

    MergeLog *t = &mergeLog; // create a shortcut
    if(logging==LoggingCriteria::SINGLE) {
        t->startTime = t->Tic();
    }

    if(logging!=LoggingCriteria::NONE) {
        // initial analysis
        t->Tic();
        double score = scoreMap();
        double st = t->Toc();
        t->updateStartTime(st);
        t->scoretime.push_back(st);
        t->scores.push_back(score);
        t->looptime.push_back(0);
        t->loopcount.push_back(0);
        t->matchtime.push_back(0);
        t->nummatches.push_back(0);
        t->maxvotes.push_back(0);
        t->fundtime.push_back(0);
        t->numedges.push_back(0);
        t->addtime.push_back(0);
        t->selecttime.push_back(0);
        t->selectedVertices.push_back(make_pair(-1,-1));
        //t->graphDistances.push_back(-1);
        //t->degrees.push_back(make_pair(-1,-1));
    }
    else
    {
        timespec ts;
        clock_gettime(CLOCK_PROCESS_CPUTIME_ID,&ts);
        mergeStartTime = ts.tv_sec + (ts.tv_nsec)/1000000000.0;
    }


    while(!done) {
        count++;
        //cout<<count<<" out of "<<A.rows<<endl;

        if(logging!=LoggingCriteria::NONE) {
            t->Tic();
        }

        vertexDescriptor v_map2 = vertex(sel,m2->getGraph()->g);
        int m2oldIndex = m2->getGraph()->g[v_map2].index;
        int m2newIndex = oldnewIndex2[m2oldIndex];
        Image *img = imageLookup[m2newIndex];


        ZointMatchMap matches1;
        int maxvote = m1->getVotes(img, matches1);
        // Due to possible index mismatches, transfer all matches using oldnewIndex of m1
        // In this current implementation it is actually NOT needed.
        ZointMatchMap matches;
        BOOST_FOREACH(ZointMatchMap::value_type zm,matches1) {
            matches[oldnewIndex1[zm.first]] = matches1[zm.first];
        }

        /*
        // ADDING ONLY ONE EDGE PER VERTEX
        ZointMatchMap matches2;
        int maxvote = m1->getVotes(img, matches2);
        ZointMatchMap matches;
        maxvote = -1;
        int maxindex = -1;
        BOOST_FOREACH(ZointMatchMap::value_type &p, matches2) {
            if((int)p.second.first.size() > maxvote) {
                maxindex = p.first;
                maxvote = p.second.first.size();
            }
        }
        matches[maxindex] = matches2[maxindex];
*/

        if(logging!=LoggingCriteria::NONE) {
            int candcount = 0;
            BOOST_FOREACH(ZointMatchMap::value_type &mm, matches) {
                ZointMatchList *ml = &(mm.second);
                if((int)(ml->first.size()) >= appConf.MINMATCHES)
                    candcount++;
            }
            //t->selectedVertices.push_back(make_pair(m2newIndex,-1));
            double st = t->Toc();
            t->matchtime.push_back(st);
            t->nummatches.push_back(candcount);
            t->maxvotes.push_back(maxvote);
            t->Tic();
        }

        CandidateList candidates;
        enforceGeometricConstraints(matches, candidates,true);


        if(logging!=LoggingCriteria::NONE) {
            double st = t->Toc();
            t->fundtime.push_back(st);
            t->numedges.push_back(candidates.size());

            /*
                    // compute largest graphDistance between newIndex & true matches
                    int maxd = -1;
                    int mindegree = 100000;
                    BOOST_FOREACH(CandidateList::value_type &c, candidates) {
                        int d=getGraph()->graphDistance(newIndex,c.first.second);
                        if (d>maxd)
                            maxd=d;
                        int deg = getGraph()->getVertexDegree(c.first.second);
                        if (deg<mindegree)
                            mindegree = deg;
                    }
                    graphDistances.push_back(maxd);
                    degrees.push_back(make_pair(getGraph()->getVertexDegree(newIndex),mindegree));
            */

            t->Tic();
        }

        getGraph()->addEdges((*(getGraph()->getVertexLookup()))[m2newIndex], candidates, imageLookup, &aaa);

        if(logging!=LoggingCriteria::NONE) {
            double st = t->Toc();
            t->addtime.push_back(st);
            t->Tic();
        }

        A.col(sel) = A.col(sel)*0.0;
        //normalizeMat(A);
        sel = selectNextVertex(A.row(sel));
        if(sel < 0) {
            sel = selectStartVertex(A);
            if(sel<0)
                done = true;
        }


        if(logging!=LoggingCriteria::NONE) {
            double st = t->Toc();
            t->selecttime.push_back(st);

            if(candidates.size()>0) {
                t->edgecount+=candidates.size();
                t->edgeaddition++;
            }

            st = t->Duration();

            t->looptime.push_back(st);
            t->loopcount.push_back(count);

            //if(t->edgeaddition > t->scoreCaptureInterval) {
            if(st > t->logTime) {

                cout<<st<<" "<<t->logTime<<" "<<t->timeInterval<<endl;

                t->edgeaddition = 0;

                t->logTime = st + t->timeInterval;



                t->Tic();
                double score = scoreMap();
                double st = t->Toc();
                t->updateStartTime(st);
                t->scoretime.push_back(st);
                t->scores.push_back(score);
                //cout<<"done in "<<st<<"secs"<<endl;
            }
            else {
                t->scoretime.push_back(0);
                t->scores.push_back(-1);
            }
        }

        if(sc==MergeStopCriteria::TIME) {
            double time;
            if(logging!=LoggingCriteria::NONE) {
                time = t->Duration();
            }
            else {
                timespec ts;
                clock_gettime(CLOCK_PROCESS_CPUTIME_ID,&ts);
                time = ts.tv_sec + (ts.tv_nsec)/1000000000.0 - mergeStartTime;
            }
            if(time > val) {
                cout<<"TIME LIMIT ("<<val<<") PASSED! Total merge time: "<<time<<endl;
                done = true;
            }
        }
        else if(sc==MergeStopCriteria::PERCENTAGE) {
            double perc = 100.0*(double)count/(double)A.rows;
            if(perc > val) {
                cout<<"PERCENTAGE LIMIT("<<val<<") PASSED! ACHIEVED: "<<perc<<"%"<<endl;
                done = true;
            }
        }
    }


    if(logging!=LoggingCriteria::NONE) {
        // final analysis
        t->looptime.push_back(t->Duration());
        t->loopcount.push_back(count+1);
        t->Tic();
        double score = scoreMap();
        double st = t->Toc();
        t->updateStartTime(st);
        t->scoretime.push_back(st);
        t->scores.push_back(score);
        t->matchtime.push_back(0);
        t->nummatches.push_back(0);
        t->maxvotes.push_back(0);
        t->fundtime.push_back(0);
        t->numedges.push_back(0);
        t->addtime.push_back(0);
        t->selecttime.push_back(0);
        t->selectedVertices.push_back(make_pair(-1,-1));
        //t->graphDistances.push_back(-1);
        //t->degrees.push_back(make_pair(-1,-1));
    }

    if(logging!=LoggingCriteria::SINGLE) {
        t->printLog();
    }

}



SimMap* SimMap::randomConnect_pairwise(SimMap** maps, int numMaps, MergeMethod::Type method, MergeStopCriteria::Type sc, double val, bool logging) {

    LoggingCriteria::Type log;
    if(!logging)
        log = LoggingCriteria::NONE;
    else {
        log = LoggingCriteria::CONTINUOUS;
        mergeLog.startTime = mergeLog.Tic();
    }

    SimMap *m;
    SimMap *m1 = maps[0];
    for(int j=1;j<numMaps;j++) {
        m = new SimMap();

        SimMap *m2 = maps[j];

        if(logging) {
            mergeLog.Tic();
        }
        map<int,int> oldnewIndex1;
        map<int,int> oldnewIndex2;
        m->unionMaps(m1,oldnewIndex1);
        m->unionMaps(m2,oldnewIndex2);
        if(logging) {
            double st = mergeLog.Toc();
            mergeLog.updateStartTime(st);
        }

        m->randomConnect(m1,m2,oldnewIndex1,oldnewIndex2,method,sc,val,log);

        if(j>1)
            delete m1;

        m1 = m;
    }

    if(logging) {
        mergeLog.printLog();
    }

    return m;
}




void SimMap::randomConnect_simultaneous(SimMap** maps, map<int,int>* oldnewIndices, int numMaps, MergeMethod::Type method, MergeStopCriteria::Type sc, double val, bool logging)
{
    Mat mapSel(1,numMaps,CV_64F, Scalar(1.0));
    mapSel.at<double>(0,0) = 0.0;
    int selectedMap = -1;

    //int selectedVertex = -1;
    int selectedVertices[numMaps];
    int startIndices[numMaps];
    Mat A[numMaps];
    for (int i=0;i<numMaps; i++) {
        startIndices[i]=-1;
        selectedVertices[i]=-1;
    }
    for (int i=0;i<numMaps; i++)
        A[i] = maps[i]->computeProbMat(method,startIndices[i]);

    bool done = false;
    int count = 0;

    MergeLog *t = &mergeLog; // create a shortcut
    if(logging) {
        t->startTime = t->Tic();

        // initial analysis
        t->Tic();
        double score = scoreMap();
        double st = t->Toc();
        t->updateStartTime(st);
        t->scoretime.push_back(st);
        t->scores.push_back(score);
        t->looptime.push_back(0);
        t->loopcount.push_back(0);
        t->matchtime.push_back(0);
        t->nummatches.push_back(0);
        t->maxvotes.push_back(0);
        t->fundtime.push_back(0);
        t->numedges.push_back(0);
        t->addtime.push_back(0);
        t->selecttime.push_back(0);
        t->selectedVertices.push_back(make_pair(-1,-1));
        //t->graphDistances.push_back(-1);
        //t->degrees.push_back(make_pair(-1,-1));
    }
    else
    {
        timespec ts;
        clock_gettime(CLOCK_PROCESS_CPUTIME_ID,&ts);
        mergeStartTime = ts.tv_sec + (ts.tv_nsec)/1000000000.0;
    }


    while(!done) {
        count++;

        bool done2=false;
        while(!done2) {

            // choose a map to sample a vertex
            selectedMap = selectNextVertex(mapSel);
            if(selectedMap<0) {
                selectedMap = selectStartVertex(mapSel);
                if(selectedMap<0) {
                    done=true;
                    break;
                }
            }

            // choose a vertex in that map
            if(selectedVertices[selectedMap]<0) { // first time...
                selectedVertices[selectedMap] = startIndices[selectedMap];
                A[selectedMap].col(selectedVertices[selectedMap]) = A[selectedMap].col(selectedVertices[selectedMap])*0.0;
                done2=true;
            }
            else {
                selectedVertices[selectedMap] = selectNextVertex(A[selectedMap].row(selectedVertices[selectedMap]));
                if(selectedVertices[selectedMap] < 0) {
                    selectedVertices[selectedMap] = selectStartVertex(A[selectedMap]);
                    if(selectedVertices[selectedMap]<0) {
                        mapSel.at<double>(0,selectedMap) = 0.0;
                    }
                }
                else {
                    A[selectedMap].col(selectedVertices[selectedMap]) = A[selectedMap].col(selectedVertices[selectedMap])*0.0;
                    done2=true;
                }
            }
        }
        if(done)
            break;


        SimMap *m2 = maps[selectedMap];
        int selectedVertex = selectedVertices[selectedMap];

        // get a vertex
        vertexDescriptor v_map2 = vertex(selectedVertex,m2->getGraph()->g);
        int m2oldIndex = m2->getGraph()->g[v_map2].index;
        int m2newIndex = oldnewIndices[selectedMap][m2oldIndex];
        Image *img = imageLookup[m2newIndex];

        for(int i=0; i<selectedMap; i++) {
            if(logging) {
                t->Tic();
            }

            SimMap *m1 = maps[i];

            ZointMatchMap matches1;
            int maxvote = m1->getVotes(img, matches1);
            // Due to possible index mismatches, transfer all matches using oldnewIndex of m1
            // In this current implementation it is actually NOT needed.
            ZointMatchMap matches;
            BOOST_FOREACH(ZointMatchMap::value_type zm,matches1) {
                matches[oldnewIndices[i][zm.first]] = matches1[zm.first];
            }

            if(logging) {
                int candcount = 0;
                BOOST_FOREACH(ZointMatchMap::value_type &mm, matches) {
                    ZointMatchList *ml = &(mm.second);
                    if((int)(ml->first.size()) >= appConf.MINMATCHES)
                        candcount++;
                }
                //t->selectedVertices.push_back(make_pair(m2newIndex,-1));
                double st = t->Toc();
                t->matchtime.push_back(st);
                t->nummatches.push_back(candcount);
                t->maxvotes.push_back(maxvote);

                t->Tic();
            }

            CandidateList candidates;
            enforceGeometricConstraints(matches, candidates,true);

            //BOOST_FOREACH(CandidateList::value_type &c, candidates) {
            //cout<<"Edge between: "<<c.first.second<<" vs "<<m2newIndex<<endl;
            //}

            if(logging) {
                double st = t->Toc();
                t->fundtime.push_back(st);
                t->numedges.push_back(candidates.size());
                t->Tic();
            }

            getGraph()->addEdges((*(getGraph()->getVertexLookup()))[m2newIndex], candidates, imageLookup, &aaa);

            if(logging) {
                double st = t->Toc();
                t->addtime.push_back(st);
                t->Tic();
            }

            if(logging) {
                double st = t->Toc();
                t->selecttime.push_back(st);

                if(candidates.size()>0) {
                    t->edgecount+=candidates.size();
                    t->edgeaddition++;
                }

                st = t->Duration();

                t->looptime.push_back(st);
                t->loopcount.push_back(count);

                //if(t->edgeaddition > t->scoreCaptureInterval) {
                if(st > t->logTime) {
                    t->edgeaddition = 0;
                    t->logTime = st + t->timeInterval;

                    t->Tic();
                    double score = scoreMap();
                    double st = t->Toc();
                    t->updateStartTime(st);
                    t->scoretime.push_back(st);
                    t->scores.push_back(score);
                    cout<<"done in "<<st<<"secs"<<endl;
                }
                else {
                    t->scoretime.push_back(0);
                    t->scores.push_back(-1);
                }
            }
        }
    }

    if(logging) {
        // final analysis
        t->looptime.push_back(t->Duration());
        t->loopcount.push_back(count+1);
        t->Tic();
        double score = scoreMap();
        double st = t->Toc();
        t->updateStartTime(st);
        t->scoretime.push_back(st);
        t->scores.push_back(score);
        t->matchtime.push_back(0);
        t->nummatches.push_back(0);
        t->maxvotes.push_back(0);
        t->fundtime.push_back(0);
        t->numedges.push_back(0);
        t->addtime.push_back(0);
        t->selecttime.push_back(0);
        t->selectedVertices.push_back(make_pair(-1,-1));
        //t->graphDistances.push_back(-1);
        //t->degrees.push_back(make_pair(-1,-1));

        t->printLog();
    }
}












// quickConnect
// Stage 1: Exploration. Word voting + adding only one edge per identified image + keep the rest of the edges for later
// Stage 2: Refinement. Adding postponed edges
void SimMap::quickConnect(SimMap *m1, SimMap *m2, map<int,int> &oldnewIndex1, map<int,int> &oldnewIndex2, MergeStopCriteria::Type sc, double val, LoggingCriteria::Type logging)
{
    boost::unordered_map<int,boost::unordered_map<int,int> > sim;
    typedef boost::unordered_map<int,bool> unorderedBoolMap;
    unorderedBoolMap isProcessedVertex1;
    unorderedBoolMap isProcessedVertex2;
    unorderedBoolMap isProcessedWord;
    vector<int> priorityList;
    vector<pair<int,int> > waitingEdges;
    int count = 0;


    // LOGGING...
    MergeLog *t = &mergeLog; // create a shortcut

    if(logging==LoggingCriteria::SINGLE) {
        t->startTime = t->Tic();
    }

    if(logging!=LoggingCriteria::NONE) {
        // initial analysis
        t->Tic();
        double score = scoreMap();
        double st = t->Toc();
        t->updateStartTime(st);
        t->scoretime.push_back(st);
        t->scores.push_back(score);
        t->looptime.push_back(0);
        t->loopcount.push_back(0);
        t->matchtime.push_back(0);
        t->nummatches.push_back(0);
        t->maxvotes.push_back(0);
        t->fundtime.push_back(0);
        t->numedges.push_back(0);
        t->addtime.push_back(0);
        t->selecttime.push_back(0);
        //t.selectedVertices.push_back(make_pair(-1,-1));
        //t.graphDistances.push_back(-1);
        //t.degrees.push_back(make_pair(-1,-1));
    }
    else {
        timespec ts;
        clock_gettime(CLOCK_PROCESS_CPUTIME_ID,&ts);
        mergeStartTime = ts.tv_sec + (ts.tv_nsec)/1000000000.0;
    }

    // SMART ALGORITHM
    // 0) Get all the words from one of the maps..
    // TODO: Need to implement tf-idf kind of weighting to sort words wrt importance
    InvIndex *indx = &(m1->getDictionary()->invIndex);
    InvIndex::iterator it = indx->begin();

    vector<int> ttt;
    while(it!=indx->end())
    {
        priorityList.push_back(it->first);
        ttt.push_back(it->first);
        it++;
    }
    while(ttt.size()>0) {
        //priorityList.push_back(ttt.back());
        ttt.pop_back();
    }
/*

    map<pair<int,int>,bool> mywords;
    while(it!=indx->end())
    {
        mywords[make_pair(it->second.size(),it->first)] = true;
        it++;
    }
    map<pair<int,int>,bool>::iterator itt = mywords.begin();
    vector<int> a,b,c;
    int cc=0;
    while(itt!=mywords.end())
    {
        if(cc > mywords.size()*0.05 && cc < mywords.size()*0.095)
            a.push_back(itt->first.second);
        cc++;
        itt++;
    }

    itt = mywords.begin();
    cc=0;
    while(itt!=mywords.end())
    {
        if(cc < mywords.size()*0.05 || cc > mywords.size()*0.095)
            b.push_back(itt->first.second);
        cc++;
        itt++;
    }
    for(int i=0; i<b.size(); i++)
        priorityList.push_back(b[i]);
    for(int i=0; i<a.size(); i++)
        priorityList.push_back(a[i]);
*/


    // STAGE 1: EXPLORATION
    cout<<"STAGE 1: Exploration"<<endl;
    while(!priorityList.empty())
    {
        int next = priorityList.back();
        priorityList.pop_back();

        // if the word is already processed, choose the next one
        if(isProcessedWord.find(next)!=isProcessedWord.end())
            continue;

        // word is not processed yet
        isProcessedWord[next]=true;
        InvIndex::iterator it1 = m1->getDictionary()->invIndex.find(next);
        InvIndex::iterator it2 = m2->getDictionary()->invIndex.find(next);

        OccuranceList *ol1 = NULL;
        OccuranceList *ol2 = NULL;
        if(it1!=m1->getDictionary()->invIndex.end())
            ol1 = &(it1->second);
        if(it2!=m2->getDictionary()->invIndex.end())
            ol2 = &(it2->second);

        if(ol1!=NULL && ol2!=NULL) {
            BOOST_FOREACH(OccuranceList::value_type &o1, *ol1) {
                BOOST_FOREACH(OccuranceList::value_type &o2, *ol2) {
                    sim[o1.first][o2.first] += o1.second.size()*o2.second.size();
                    //sim2[o1.first][o2.first] += o1.second.size()*o2.second.size();
                    //movie<<o1.first<<" "<<o2.first<<" "<<o1.second.size()*o2.second.size()<<endl;

                    if(sim[o1.first][o2.first] >= appConf.MINMATCHES) {
                        sim[o1.first][o2.first] = -10000; // thanks to this trick I dont need to keep track of postponed edges

                        int m1newIndex = oldnewIndex1[o1.first];
                        int m2newIndex = oldnewIndex2[o2.first];

                        if(isProcessedVertex1.find(m1newIndex)!=isProcessedVertex1.end() && isProcessedVertex2.find(m2newIndex)!=isProcessedVertex2.end()) {
                        //if(isProcessedVertex2.find(m2newIndex)!=isProcessedVertex2.end()) {
                            waitingEdges.push_back(make_pair(m1newIndex,m2newIndex));
                        }
                        else {
                            // trying to insert an identified edge

                            isProcessedVertex1[m1newIndex] = true; //proccessed image o1.first
                            isProcessedVertex2[m2newIndex] = true; //proccessed image o2.first

                            count++;
                            //cout<<count<<" out of "<<m2->numVertices()<<endl;

                            if(logging!=LoggingCriteria::NONE) {
                                t->Tic();
                            }

                            Image *img1 = imageLookup[m1newIndex];
                            Image *img2 = imageLookup[m2newIndex];


                            // WHAT ABOUT RANDOM SELECTION OF WORDS FROM PRIORITYLIST??
                            for(int wi=0; wi<img1->words.size(); wi++) {
                                // add only if not processed yet
                                int candword = img1->words[wi].first;
                                if(isProcessedWord.find(candword)==isProcessedWord.end())
                                    priorityList.push_back(candword);
                            }
                            for(int wi=0; wi<img2->words.size(); wi++) {
                                // add only if not processed yet
                                int candword = img2->words[wi].first;
                                if(isProcessedWord.find(candword)==isProcessedWord.end())
                                    priorityList.push_back(candword);
                            }

                            ZointMatchMap matches;
                            //int maxvote = m1->getVotes(img2, matches);

                            // idea: instead of adding all only add one edge per image
                            int maxvote = 0;
                            int M = (int)img2->words.size();
                            for(int i=0; i<M; i++) { // for each word in img2
                                BOOST_FOREACH(ZointList::value_type &p, getDictionary()->invIndex[img2->words[i].first][m1newIndex]) { // take the occurances of the same words in image1
                                    matches[m1newIndex].first.push_back(img2->words[i].second);
                                    matches[m1newIndex].second.push_back(p);
                                    maxvote++;
                                }
                            }

                            /*
                            BOOST_FOREACH(ZointMatchMap::value_type &o, matches) {
                                if(o.second.first.size()>= appConf.MINMATCHES) {
                                    cout<<"Match between: "<<m1newIndex<<" vs "<<m2newIndex<<" with "<<o.second.first.size()<<endl; //m1 vs m2
                                    for(int u=0;u<o.second.first.size();u++) {
                                        cout<<"Point: "<<o.second.first[u]<<" "<<o.second.second[u]<<endl;
                                    }
                                }
                            }
                            */

                            if(logging!=LoggingCriteria::NONE) {
                                double st = t->Toc();
                                t->matchtime.push_back(st);
                                t->nummatches.push_back(matches.size());
                                t->maxvotes.push_back(maxvote);

                                t->Tic();
                            }

                            CandidateList candidates;
                            enforceGeometricConstraints(matches, candidates,true);

                            if(logging!=LoggingCriteria::NONE) {
                                double st = t->Toc();
                                t->fundtime.push_back(st);
                                t->numedges.push_back(candidates.size());

                                //cout<<"img "<<o1.first<<"("<<img1->numFeatures()<<") vs "<<o2.first<<"("<<img2->numFeatures()<<") with "<<maxvote<<" in "<<t.getTime()<<" sec"<<endl;

                                /*
                                // compute the graphDistance between o1.first & newIndex
                                int d=getGraph()->graphDistance(o1.first,newIndex);
                                int deg1 = getGraph()->getVertexDegree(o1.first);
                                int deg2 = getGraph()->getVertexDegree(newIndex);
                                t.graphDistances.push_back(d);
                                t.degrees.push_back(make_pair(deg1,deg2));
                                t.selectedVertices.push_back(make_pair(o1.first,o2.first));
                                */

                                t->Tic();
                            }


                            //BOOST_FOREACH(CandidateList::value_type &o, candidates) {
                            //    cout<<"Edge between: "<<o.first.second<<" vs "<<m2newIndex<<endl; //m1 vs m2
                            //}

                            getGraph()->addEdges((*(getGraph()->getVertexLookup()))[m2newIndex], candidates, imageLookup,&aaa);

                            if(logging!=LoggingCriteria::NONE) {
                                double st = t->Toc();
                                t->addtime.push_back(st);
                                t->selecttime.push_back(0);

                                if(candidates.size()>0) {
                                    t->edgeaddition++;
                                    t->edgecount+=candidates.size();
                                }

                                t->looptime.push_back(t->Duration());
                                t->loopcount.push_back(count);

                                if(t->edgeaddition > t->explorationScoreCaptionInterval) {
                                    t->edgeaddition = 0;
                                    t->Tic();
                                    double score = scoreMap();
                                    double st = t->Toc();
                                    t->updateStartTime(st);
                                    t->scoretime.push_back(st);
                                    t->scores.push_back(score);
                                }
                                else {
                                    t->scoretime.push_back(0);
                                    t->scores.push_back(-1);
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    if(logging!=LoggingCriteria::NONE) {
        double st = t->Duration();
        cout<<"time: "<<st<<endl;

        // write down zeros so that we can identify when exploration ends and when refinement starts
        t->looptime.push_back(st);
        t->loopcount.push_back(0);
        t->matchtime.push_back(0);
        t->nummatches.push_back(0);
        t->maxvotes.push_back(0);
        t->fundtime.push_back(0);
        t->numedges.push_back(0);
        t->addtime.push_back(0);
        t->selecttime.push_back(0);
        //t->degrees.push_back(make_pair(-1,-1));
        //t->graphDistances.push_back(-1);
        //t->selectedVertices.push_back(make_pair(-1,-1));
        t->Tic();
        double score = scoreMap();
        st = t->Toc();
        t->updateStartTime(st);
        t->scoretime.push_back(st);
        t->scores.push_back(score);
    }
    cout<<"Count: "<<count<<endl;

    aaa<<0<<"\t"<<0<<endl;

    // STAGE 2: REFINEMENT
    cout<<"STAGE 2: Refinement"<<endl;
    // Sort edges wrt mindegree
    typedef map<pair<int,pair<int,int> >, bool> mysortedEdges;
    mysortedEdges sortedEdges;
    for(int j=0; j<waitingEdges.size();j++) {
        int i1 = waitingEdges[j].first; // new index
        int i2 = waitingEdges[j].second; // new index
        int mindegree = min(degree((*(getGraph()->getVertexLookup()))[i1],getGraph()->g),degree((*(getGraph()->getVertexLookup()))[i2],getGraph()->g));
        sortedEdges[make_pair(mindegree, make_pair(i1,i2))] = true;
    }
    cout<<"size of waiting list: "<<waitingEdges.size()<<endl;

    if(true) {
        // NOW PROCESS REMAINING OF THE EDGES
        BOOST_FOREACH(mysortedEdges::value_type &ee, sortedEdges) {

            count++;

            if(logging!=LoggingCriteria::NONE) {
                t->looptime.push_back(t->Duration());
                t->loopcount.push_back(count);
                t->Tic();
            }

            //int m1newIndex = waitingEdges[j].first;
            //int m2oldIndex = waitingEdges[j].second;
            int m1newIndex = ee.first.second.first;
            int m2newIndex = ee.first.second.second;
            Image *img1 = imageLookup[m1newIndex];
            Image *img2 = imageLookup[m2newIndex];

            ZointMatchMap matches;

            int maxvote = 0;
            int M = (int)img2->words.size();
            for(int i=0; i<M; i++) {
                BOOST_FOREACH(ZointList::value_type &p, getDictionary()->invIndex[img2->words[i].first][m1newIndex]) {
                    matches[m1newIndex].first.push_back(img2->words[i].second);
                    matches[m1newIndex].second.push_back(p);
                    maxvote++;
                }
            }


            if(logging!=LoggingCriteria::NONE) {
                double st = t->Toc();
                t->matchtime.push_back(st);
                t->nummatches.push_back(matches.size());
                t->maxvotes.push_back(maxvote);

                t->Tic();
            }

            CandidateList candidates;
            enforceGeometricConstraints(matches, candidates,true);

            if(logging!=LoggingCriteria::NONE) {
                double st = t->Toc();
                t->fundtime.push_back(st);
                t->numedges.push_back(candidates.size());

                /*
                // compute the graphDistance between o1.first & newIndex
                int d=getGraph()->graphDistance(m1newIndex,m2newIndex);
                int deg1 = getGraph()->getVertexDegree(m1newIndex);
                int deg2 = getGraph()->getVertexDegree(m2newIndex);
                graphDistances.push_back(d);
                degrees.push_back(make_pair(deg1,deg2));
                selectedVertices.push_back(make_pair(m1newIndex,m2newIndex));
                */

                t->Tic();
            }

            //BOOST_FOREACH(CandidateList::value_type &o, candidates) {
            //    cout<<"Edge between: "<<o.first.second<<" vs "<<m2newIndex<<endl; //m1 vs m2
            //}

            getGraph()->addEdges((*(getGraph()->getVertexLookup()))[m2newIndex], candidates, imageLookup,&aaa);

            if(logging!=LoggingCriteria::NONE) {
                double st = t->Toc();
                t->addtime.push_back(st);
                t->selecttime.push_back(0);

                if(candidates.size()>0) {
                    t->edgeaddition++;
                    t->edgecount+=candidates.size();
                }


                if(t->edgeaddition > t->refinementScoreCaptionInterval) {
                    t->edgeaddition = 0;
                    t->Tic();
                    double score = scoreMap();
                    double st = t->Toc();
                    t->updateStartTime(st);
                    t->scoretime.push_back(st);
                    t->scores.push_back(score);
                }
                else {
                    t->scoretime.push_back(0);
                    t->scores.push_back(-1);
                }
            }
        }
    }

    if(logging!=LoggingCriteria::NONE) {
        // final analysis
        t->looptime.push_back(t->Duration());
        t->loopcount.push_back(count+1);
        t->Tic();
        double score = scoreMap();
        double st = t->Toc();
        t->updateStartTime(st);
        t->scoretime.push_back(st);
        t->scores.push_back(score);
        t->matchtime.push_back(0);
        t->nummatches.push_back(0);
        t->maxvotes.push_back(0);
        t->fundtime.push_back(0);
        t->numedges.push_back(0);
        t->addtime.push_back(0);
        t->selecttime.push_back(0);
        //t.selectedVertices.push_back(make_pair(-1,-1));
        //t.degrees.push_back(make_pair(-1,-1));
        //t.graphDistances.push_back(-1);
    }
    if(logging==LoggingCriteria::SINGLE) {
        t->printLog();
    }

    aaa.close();
}



SimMap* SimMap::quickConnect_pairwise(SimMap** maps, int numMaps, MergeStopCriteria::Type sc, double val, bool logging)
{
    LoggingCriteria::Type log;
    if(!logging)
        log = LoggingCriteria::NONE;
    else {
        log = LoggingCriteria::CONTINUOUS;
        mergeLog.startTime = mergeLog.Tic();
    }

    SimMap *m;
    SimMap *m1 = maps[0];
    for(int j=1;j<numMaps;j++) {
        m = new SimMap();

        SimMap *m2 = maps[j];

        if(logging) {
            mergeLog.Tic();
        }
        map<int,int> oldnewIndex1;
        map<int,int> oldnewIndex2;
        m->unionMaps(m1,oldnewIndex1);
        m->unionMaps(m2,oldnewIndex2);
        if(logging) {
            double st = mergeLog.Toc();
            mergeLog.updateStartTime(st);
        }

        m->quickConnect(m1,m2,oldnewIndex1,oldnewIndex2,sc,val,log);

        if(j>1)
            delete m1;

        m1 = m;
    }

    if(logging) {
        mergeLog.printLog();
    }

    return m;
}




// quickConnect for multi map merging
// Stage 1: Exploration. Word voting + adding only one edge per identified image + keep the rest of the edges for later
// Stage 2: Refinement. Adding postponed edges
void SimMap::quickConnect_simultaneous(SimMap** maps, map<int,int> *oldnewIndex, int numMaps, MergeStopCriteria::Type sc, double val, bool logging)
{
    /*
    // number of 2-combinations of numMaps: for each combination save a map<intIndex,map<intIndex,intCount>>
    // how to keep track of the sims?
    // if there is a similarity between map x & y, what is the index of the simMatrix that should be used?

    n! / (n-m)! m! or (numberOfFaces in n dimensional hypercube/2^(n-2))

    first one is gonna use: n-1 slots
    second one is gonna use: n-2 slots
    third one is gonna use: n-3 slots

    cum:
    1: 0
    2: n-1
    3: n-1 + n-2
    4: n-1 + n-2 + n-3

    index (start from 1): (x-1)*n - ((x-1)*(x))/2  + y - x:
    example: 1&2 = 0-0 + 2-1 = 1
    example: 1&4 = 0-0 + 4-1 = 3
    example: 2&4 = 4-1 + 4-2 = 5
    example: 3&4 = 8 - 2*3/2 + 4 - 3 = 6

    index (start from 0): (x)*n - ((x+1)*(x))/2  + y - x - 1:
    example: 0&1 = 0-0 + 1-0-1 = 0
    example: 0&3 = 0-0 + 3-0-1 = 2
    example: 1&3 = 4-2*1/2 + 3-1-1 = 4
    example: 2&3 = 8 - 2*3/2 + 3 - 2 - 1 = 5

    1 1-2: 2
    2 1-3: 3
    3 1-4: 4
    4 2-3: 6
    5 2-4: 8
    6 3-4: 12
    // create a map<int,int> to hold indices and lookup with x*y????

    m1  xxx
    m2  xxxxx
    m3  xx
    m4  xxxxxxxxxxxx
    */


    // LOGGING...
    MergeLog *t = &mergeLog; // create a shortcut

    if(logging) {
        t->startTime = t->Tic();

        // initial analysis
        t->Tic();
        double score = scoreMap();
        double st = t->Toc();
        t->updateStartTime(st);
        t->scoretime.push_back(st);
        t->scores.push_back(score);
        t->looptime.push_back(0);
        t->loopcount.push_back(0);
        t->matchtime.push_back(0);
        t->nummatches.push_back(0);
        t->maxvotes.push_back(0);
        t->fundtime.push_back(0);
        t->numedges.push_back(0);
        t->addtime.push_back(0);
        t->selecttime.push_back(0);
    }


    // Create all map combinations
    typedef map<int,pair<int,int> > CombinationMap; // <combinationIndex, <map1,map2> >
    CombinationMap combinations;
    int c=0;
    for (int i=0; i<numMaps-1; i++) {
        for (int j=i+1; j<numMaps; j++) {
            combinations[c++]=make_pair(i,j);
        }
    }

    // Create all similarity matrices for all combinations
    typedef boost::unordered_map<int,boost::unordered_map<int,int> > IntIntIntMap; // <v1, <v2, vote>>
    IntIntIntMap sims[combinations.size()];


    boost::unordered_map<int,bool> isProcessedWord;
    IntIntIntMap vertexConnections; // with new vertexId <newVertexId, Id of the targeting map, numEdges to that map>
    vector<pair<int,int> > waitingEdges; // with new vertexId
    vector<int> priorityList;
    int count = 0;

    // SMART ALGORITHM
    // STEP 0: Get all the words
    int numWords = getDictionary()->numWords();
    for(int i=numWords-1; i>=0; i--) {
        priorityList.push_back(i);
    }


    // STEP 1: EXPLORATION
    cout<<"STEP 1: Exploration"<<endl;
    while(!priorityList.empty())
    {
        int next = priorityList.back();
        priorityList.pop_back();

        // if the word is already processed, choose the next one
        if(isProcessedWord.find(next)!=isProcessedWord.end())
            continue;

        // word is not processed yet
        isProcessedWord[next]=true;


        for(int i=0; i<combinations.size(); i++)
        {
            int m1 = combinations[i].first;
            int m2 = combinations[i].second;

            InvIndex::iterator it1 = maps[m1]->getDictionary()->invIndex.find(next);
            InvIndex::iterator it2 = maps[m2]->getDictionary()->invIndex.find(next);

            OccuranceList *ol1 = NULL;
            OccuranceList *ol2 = NULL;
            if(it1!=maps[m1]->getDictionary()->invIndex.end())
                ol1 = &(it1->second);
            if(it2!=maps[m2]->getDictionary()->invIndex.end())
                ol2 = &(it2->second);

            if(ol1!=NULL && ol2!=NULL) {
                BOOST_FOREACH(OccuranceList::value_type &o1, *ol1) {
                    BOOST_FOREACH(OccuranceList::value_type &o2, *ol2) {

                        int i1oldIndex = o1.first;
                        int i2oldIndex = o2.first;
                        int i1newIndex = oldnewIndex[m1][i1oldIndex];
                        int i2newIndex = oldnewIndex[m2][i2oldIndex];

                        sims[i][i1newIndex][i2newIndex] += o1.second.size()*o2.second.size();

                        if(sims[i][i1newIndex][i2newIndex] >= appConf.MINMATCHES) {
                            sims[i][i1newIndex][i2newIndex] = -1000; // thanks to this trick I dont need to keep track of postponed edges

                            //cout<<"Edge identified between: "<<m1<<" and "<<m2<<endl;

                            //if(isProcessedVertex.find(i1new)!=isProcessedVertex.end() || isProcessedVertex.find(i2new)!=isProcessedVertex.end()) {
                            if(vertexConnections[i1newIndex][m2]>0 && vertexConnections[i2newIndex][m1]>0) {
                                waitingEdges.push_back(make_pair(i1newIndex,i2newIndex));
                            }
                            else {
                                // trying to insert an identified edge

                                count++;

                                if(logging) {
                                    t->Tic();
                                }


                                //isProcessedVertex[i1newIndex] = true; //proccessed image o1.first
                                //isProcessedVertex[i2newIndex] = true; //proccessed image o2.first
                                vertexConnections[i1newIndex][m2]++;
                                vertexConnections[i2newIndex][m1]++;


                                Image *img1 = imageLookup[i1newIndex];
                                Image *img2 = imageLookup[i2newIndex];

                                // WHAT ABOUT RANDOM SELECTION OF WORDS FROM PRIORITYLIST??
                                for(int wi=0; wi<img1->words.size(); wi++) {
                                    // add only if not processed yet
                                    int candword = img1->words[wi].first;
                                    if(isProcessedWord.find(candword)==isProcessedWord.end())
                                        priorityList.push_back(candword);
                                }
                                for(int wi=0; wi<img2->words.size(); wi++) {
                                    // add only if not processed yet
                                    int candword = img2->words[wi].first;
                                    if(isProcessedWord.find(candword)==isProcessedWord.end())
                                        priorityList.push_back(candword);
                                }


                                ZointMatchMap matches;

                                // idea: instead of adding all only add one edge per image
                                // Create a list of feature matches between img1 and img2 from img1 perspective
                                int maxvote = 0;
                                int M = (int)img2->words.size();
                                for(int i=0; i<M; i++) { // for each word in image1
                                    BOOST_FOREACH(ZointList::value_type &p, getDictionary()->invIndex[img2->words[i].first][i1newIndex]) {
                                        // take the occurances of the same words in image2
                                        matches[i1newIndex].first.push_back(img2->words[i].second);
                                        matches[i1newIndex].second.push_back(p);
                                        maxvote++;
                                    }
                                }


                                if(logging) {
                                    double st = t->Toc();
                                    t->matchtime.push_back(st);
                                    t->nummatches.push_back(matches.size());
                                    t->maxvotes.push_back(maxvote);

                                    t->Tic();
                                }

                                CandidateList candidates;
                                enforceGeometricConstraints(matches, candidates,true);

                                if(logging) {
                                    double st = t->Toc();
                                    t->fundtime.push_back(st);
                                    t->numedges.push_back(candidates.size());

                                    t->Tic();
                                }

                                //BOOST_FOREACH(CandidateList::value_type &o, candidates) {
                                //    cout<<"Edge between: "<<i1newIndex<<" vs "<<o.first.second<<endl; //m1 vs m2
                                //}

                                getGraph()->addEdges((*(getGraph()->getVertexLookup()))[i2newIndex], candidates, imageLookup, &aaa);

                                if(logging) {
                                    double st = t->Toc();
                                    t->addtime.push_back(st);
                                    t->selecttime.push_back(0);

                                    if(candidates.size()>0) {
                                        t->edgeaddition++;
                                        t->edgecount+=candidates.size();
                                    }

                                    t->looptime.push_back(t->Duration());
                                    t->loopcount.push_back(count);

                                    if(t->edgeaddition > t->explorationScoreCaptionInterval) {
                                        t->edgeaddition = 0;
                                        t->Tic();
                                        double score = scoreMap();
                                        //cout<<"v1 v2 e: "<<o1.first<<" "<<o2.first<<" "<<candidates.size()<<" "<<score<<endl;
                                        double st = t->Toc();
                                        t->updateStartTime(st);
                                        t->scoretime.push_back(st);
                                        t->scores.push_back(score);
                                    }
                                    else {
                                        t->scoretime.push_back(0);
                                        t->scores.push_back(-1);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    if(logging) {
        double st = t->Duration();
        cout<<"time: "<<st<<endl;

        // write down zeros so that we can identify when exploration ends and when refinement starts
        t->looptime.push_back(st);
        t->loopcount.push_back(0);
        t->matchtime.push_back(0);
        t->nummatches.push_back(0);
        t->maxvotes.push_back(0);
        t->fundtime.push_back(0);
        t->numedges.push_back(0);
        t->addtime.push_back(0);
        t->selecttime.push_back(0);
        t->Tic();
        double score = scoreMap();
        st = t->Toc();
        t->updateStartTime(st);
        t->scoretime.push_back(st);
        t->scores.push_back(score);
    }




    // STEP 2: REFINEMENT
    cout<<"STEP 2: Refinement"<<endl;
    // Sort edges wrt mindegree
    typedef map<pair<int,pair<int,int> >, bool> mysortedEdges;
    mysortedEdges sortedEdges;
    for(int j=0; j<waitingEdges.size();j++) {
        int i1 = waitingEdges[j].first;
        int i2 = waitingEdges[j].second;
        int mindegree = min(degree((*(getGraph()->getVertexLookup()))[i1],getGraph()->g),degree((*(getGraph()->getVertexLookup()))[i2],getGraph()->g));
        sortedEdges[make_pair(mindegree, make_pair(i1,i2))] = true;
    }
    cout<<"size of waiting list: "<<waitingEdges.size()<<endl;


    if(true) {
        // NOW PROCESS REMAINING OF THE EDGES
        BOOST_FOREACH(mysortedEdges::value_type &ee, sortedEdges) {
            count++;

            if(logging) {
                t->looptime.push_back(t->Duration());
                t->loopcount.push_back(count);

                t->Tic();
            }


            int i1newIndex = ee.first.second.first;
            int i2newIndex = ee.first.second.second;
            Image *img1 = imageLookup[i1newIndex];
            Image *img2 = imageLookup[i2newIndex];

            ZointMatchMap matches;

            int maxvote = 0;
            int M = (int)img2->words.size();
            for(int i=0; i<M; i++) {
                BOOST_FOREACH(ZointList::value_type &p, getDictionary()->invIndex[img2->words[i].first][i1newIndex]) {
                    matches[i1newIndex].first.push_back(img2->words[i].second);
                    matches[i1newIndex].second.push_back(p);
                    maxvote++;
                }
            }

            //BOOST_FOREACH(ZointMatchMap::value_type &o, matches) {
            //    if(o.second.first.size()>= appConf.MINMATCHES)
            //        cout<<"Match between: "<<i1newIndex<<" vs "<<i2newIndex<<" with "<<o.second.first.size()<<endl; //m1 vs m2
            //}


            if(logging) {
                double st = t->Toc();
                t->matchtime.push_back(st);
                t->nummatches.push_back(matches.size());
                t->maxvotes.push_back(maxvote);

                t->Tic();
            }

            CandidateList candidates;
            enforceGeometricConstraints(matches, candidates,true);

            if(logging) {
                double st = t->Toc();
                t->fundtime.push_back(st);
                t->numedges.push_back(candidates.size());

                t->Tic();
            }

            //BOOST_FOREACH(CandidateList::value_type &o, candidates) {
            //    cout<<"Edge between: "<<i1newIndex<<" vs "<<o.first.second<<endl; //m1 vs m2
            //}

            getGraph()->addEdges((*(getGraph()->getVertexLookup()))[i2newIndex], candidates, imageLookup);

            if(logging) {
                if(candidates.size()>0) {
                    t->edgeaddition++;
                    t->edgecount+=candidates.size();
                }

                //cout<<"count: "<<count<<"  edgeaddition: "<<edgeaddition<<endl;

                double st = t->Toc();
                t->addtime.push_back(st);
                t->selecttime.push_back(0);

                if(t->edgeaddition > t->refinementScoreCaptionInterval) {
                    t->edgeaddition = 0;
                    t->Tic();
                    double score = scoreMap();
                    double st = t->Toc();
                    t->updateStartTime(st);
                    t->scoretime.push_back(st);
                    t->scores.push_back(score);
                }
                else {
                    t->scoretime.push_back(0);
                    t->scores.push_back(-1);
                }
            }

        }
    }

    if(logging) {
        // final analysis
        t->looptime.push_back(t->Duration());
        t->loopcount.push_back(count+1);
        t->Tic();
        double score = scoreMap();
        double st = t->Toc();
        t->updateStartTime(st);
        t->scoretime.push_back(st);
        t->scores.push_back(score);
        t->matchtime.push_back(0);
        t->nummatches.push_back(0);
        t->maxvotes.push_back(0);
        t->fundtime.push_back(0);
        t->numedges.push_back(0);
        t->addtime.push_back(0);
        t->selecttime.push_back(0);

        t->printLog();
    }
}


















// dump graph data to file in order to find the minimal algCon gain for addition of 1 edge
void SimMap::dumpGraph(SimMap *m1, SimMap *m2,map<int,int> &oldnewIndex)
{
    // ******************** TEMP START *************************
    const int n = num_vertices(getGraph()->g);

    // Create d matrix and a list of all possible edges
    Mat d(n,n,CV_32F);
    pair<vertexIterator,vertexIterator> vrange = vertices(getGraph()->g);
    int i=0;
    for(vertexIterator v1=vrange.first; v1!=vrange.second; v1++) {
        int j=0;
        for(vertexIterator v2=vrange.first; v2!=vrange.second; v2++) {
            if(i==j) {
                //d.at<float>(i,j) = degree(*v1,getGraph()->g); // not really! :)
                //since I am removing edges I should also remove them during degree computations
                pair<outedgeIterator, outedgeIterator> outrange = out_edges(*v1,getGraph()->g);
                pair<inedgeIterator, inedgeIterator> inrange = in_edges(*v1,getGraph()->g);
                outedgeIterator it1;
                inedgeIterator it2;

                int deg = 0;
                for(it1=outrange.first; it1!=outrange.second; it1++) {
                    if(getGraph()->gEdge[*it1].customVar>=0)
                        deg++;
                }
                for(it2=inrange.first; it2!=inrange.second; it2++) {
                    if(getGraph()->gEdge[*it2].customVar>=0)
                        deg++;
                }
                d.at<float>(i,j) = deg/2;
            }
            else {
                edgeDescr edgebool = edge(*v1,*v2,getGraph()->g);
                if(edgebool.second) {
                    if(getGraph()->gEdge[edgebool.first].customVar < 0)
                        d.at<float>(i,j) = 0;
                    else
                        d.at<float>(i,j) = -1;
                }
                else
                    d.at<float>(i,j) = 0;
            }
            j++;
        }
        i++;
    }


    // Create a list of all possible edges
    map<pair<int,int>,bool> edgelist;
    //randomConnect(m1, m2, oldnewIndex, MergeMethod::BRUTEORDER, MergeStopCriteria::PERCENTAGE, 200);
    updateStats();
    vrange = vertices(getGraph()->g);
    i=0;
    for(vertexIterator v1=vrange.first; v1!=vrange.second; v1++) {
        int j=0;
        for(vertexIterator v2=vrange.first; v2!=vrange.second; v2++) {
            if(i!=j) {
                if(getGraph()->g[*v1].mapId != getGraph()->g[*v2].mapId) {
                    edgeDescr edgebool = edge(*v1,*v2,getGraph()->g);
                    if(edgebool.second) {
                        if(getGraph()->gEdge[edgebool.first].customVar >= 0) {
                            edgelist[make_pair(min(i,j),max(i,j))] = true;
                        }
                        else
                            cout<<"false edge"<<endl;
                    }
                }
            }
            j++;
        }
        i++;
    }


    ofstream out1("_d.txt");
    out1<<n<<" "<<n<<endl;
    for(int i=0; i<n; i++) {
        for(int j=0; j<n; j++) {
            out1<<d.at<float>(i,j)<<" ";
        }
        out1<<endl;
    }
    out1.close();

    ofstream out2("_e.txt");
    map<pair<int,int>,bool>::iterator it;
    for(it=edgelist.begin(); it!=edgelist.end(); it++) {
        if(it->second) {
            out2<<it->first.first<<" "<<it->first.second<<endl;
        }
    }
    out2.close();;
}


void SimMap::dumpGraph(ofstream &out, double time, SimMap *m1, SimMap *m2, map<int,int> &oldnewIndex) {
    out<<"-----------------------"<<endl;
    out<<time<<":"<<endl;
    pair<vertexIterator,vertexIterator> vrange1 = vertices(m1->getGraph()->g);
    pair<vertexIterator,vertexIterator> vrange2 = vertices(m2->getGraph()->g);

    int i1,i2;
    for(vertexIterator v1=vrange1.first; v1!=vrange1.second; v1++) {
        // m1 vs m1
        for(vertexIterator v2=vrange1.first; v2!=vrange1.second; v2++) {
            i1 = m1->getGraph()->g[*v1].index;
            i2 = m1->getGraph()->g[*v2].index;
            if(i1!=i2) {
                pair<edgeDescriptor,bool> edgebool = edge((*(getGraph()->getVertexLookup()))[i1],(*(getGraph()->getVertexLookup()))[i2],getGraph()->g);
                if(edgebool.second)
                    out<<getGraph()->gEdge[edgebool.first].weight<<" ";
                else
                    out<<"0 ";
            }
            else
                out<<"0 ";
        }
        // m1 vs m2
        for(vertexIterator v2=vrange2.first; v2!=vrange2.second; v2++) {
            i1 = m1->getGraph()->g[*v1].index;
            i2 = oldnewIndex[m2->getGraph()->g[*v2].index];
            if(i1!=i2) {
                pair<edgeDescriptor,bool> edgebool = edge((*(getGraph()->getVertexLookup()))[i1],(*(getGraph()->getVertexLookup()))[i2],getGraph()->g);
                if(edgebool.second)
                    out<<getGraph()->gEdge[edgebool.first].weight<<" ";
                else
                    out<<"0 ";
            }
            else
                out<<"0 ";
        }
        out<<endl;
    }
    for(vertexIterator v1=vrange2.first; v1!=vrange2.second; v1++) {
        // m2 vs m1
        for(vertexIterator v2=vrange1.first; v2!=vrange1.second; v2++) {
            i1 = oldnewIndex[m2->getGraph()->g[*v1].index];
            i2 = m1->getGraph()->g[*v2].index;
            if(i1!=i2) {
                pair<edgeDescriptor,bool> edgebool = edge((*(getGraph()->getVertexLookup()))[i1],(*(getGraph()->getVertexLookup()))[i2],getGraph()->g);
                if(edgebool.second)
                    out<<getGraph()->gEdge[edgebool.first].weight<<" ";
                else
                    out<<"0 ";
            }
            else
                out<<"0 ";
        }
        // m2 vs m2
        for(vertexIterator v2=vrange2.first; v2!=vrange2.second; v2++) {
            i1 = oldnewIndex[m2->getGraph()->g[*v1].index];
            i2 = oldnewIndex[m2->getGraph()->g[*v2].index];
            if(i1!=i2) {
                pair<edgeDescriptor,bool> edgebool = edge((*(getGraph()->getVertexLookup()))[i1],(*(getGraph()->getVertexLookup()))[i2],getGraph()->g);
                if(edgebool.second)
                    out<<getGraph()->gEdge[edgebool.first].weight<<" ";
                else
                    out<<"0 ";
            }
            else
                out<<"0 ";
        }
        out<<endl;
    }
}


void SimMap::dumpGraphHeader(ofstream &out, SimMap *m1, SimMap *m2) {
    out<<"m1: "<<m1->getGraph()->numVertices()<<endl;
    out<<"m2: "<<m2->getGraph()->numVertices()<<endl;
    out<<"-----------------------"<<endl;

    out<<"positions (x,y):"<<endl;
    pair<vertexIterator,vertexIterator> vrange1 = vertices(m1->getGraph()->g);
    pair<vertexIterator,vertexIterator> vrange2 = vertices(m2->getGraph()->g);

    for(vertexIterator v1=vrange1.first; v1!=vrange1.second; v1++) {
        out<<m1->getGraph()->g[*v1].gridmapPose.center.x<<" "<<m1->getGraph()->g[*v1].gridmapPose.center.y<<endl;
    }
    for(vertexIterator v2=vrange2.first; v2!=vrange2.second; v2++) {
        out<<m2->getGraph()->g[*v2].gridmapPose.center.x<<" "<<m2->getGraph()->g[*v2].gridmapPose.center.y<<endl;
    }
}


